#include <sstream>
#include <algorithm>
#include <iostream>
#include <string>
#include <vector>
#include <memory>
#include <functional>
#include "CodeGen.h"
// #include "SSA.cpp"
#include "../util/node.h"
#include "../util/type.h"

using namespace compiler::node;
namespace type = compiler::type;

char const *last_arm_gen_erro = nullptr;

static int registers[10] = { };
const int reg_len = 10;

int reg_allocate(int reg, std::ostringstream &os) {
	if (registers[reg]++)
		os << "	push	{r" << reg << "}" << std::endl;
	return reg;
}

int reg_allocate(std::ostringstream &os) {
	for (int i = reg_len - 1; i >= 0; --i) {
		if (!registers[i]) {
			++registers[i];
			return i;
		}
	}
	return reg_allocate(9, os);
}

void reg_deallocate(int reg, std::ostringstream &os) {
	if (--registers[reg])
		os << "	pop	{r" << reg << "}" << std::endl;
}

int is_valid_immediate_value(int _value, int &new_value) {
	unsigned int value = _value;
	if (_value < 0)
		value = ~_value;
	for (int i = 0; i < 32; i += 2) {
		if (value < 256) {
			new_value = value;
			return i;
		}
		value = value << 2 | value >> 30;
	}
	return -1;
}
inline bool is_valid_immediate_value(int _value) {
	int new_value;
	return -1 < is_valid_immediate_value(_value, new_value);
}

bool get_target_var_offset(std::shared_ptr<VarTypeNode> const &target_var, int &offset) {
	std::shared_ptr<Node> const &_parent = target_var->get_parent();
	if (_parent->get_type() != type::NODE_TYPE:: N_VAR_DEF) {
		last_arm_gen_erro = "The parent of the target variable must be of type VarDefNode!";
		return false;
	}
	std::shared_ptr<VarFuncNode> const &parent = std::dynamic_pointer_cast<VarFuncNode>(_parent);
	std::vector<std::shared_ptr<VarTypeNode> > const &var_type = parent->get_var_type();
	int index = 0;
	for (int i = 0; i < var_type.size(); ++i) {
		if (var_type[i] == target_var) {
			offset = parent->get_off_set()[i];
			return true;
		}
	}
	last_arm_gen_erro = "No matching target variable was found!";
	return false;
}

template<class NODE_TYPE>
bool arm_gen_array(std::vector<std::shared_ptr<NODE_TYPE>> &nodes, std::ostringstream &oss) {
	for (typename std::vector<std::shared_ptr<NODE_TYPE>>::iterator it = nodes.begin(), end = nodes.end(); it != end; ++it) {
		if (!arm_gen(std::dynamic_pointer_cast<Node>(*it), oss))
			return false;
	}
	return true;
}

template<typename OPERATE>
inline long long calculation(long long operand0, long long operand1, OPERATE operate) {
	return operate(operand0, operand1);
}

template<typename OPERATE>
bool calculation(const std::vector<std::shared_ptr<Node>>& parameters, const std::shared_ptr<Node>& parent, OPERATE operate, std::shared_ptr<LiteralNode>& literal) {
    std::shared_ptr<LiteralNode> operand0;
    std::shared_ptr<LiteralNode> operand1;
    if (!check_literal(parameters[0], operand0))
        return false;
    if (!check_literal(parameters[1], operand1))
        return false;
    literal = std::make_shared<LiteralNode>(parent, type::NODE_TYPE::N_LITERAL, calculation(operand0->get_value(), operand1->get_value(), operate));
    return true;
}


std::function<long long(long long, long long)> get_operation_function(type::OPERATION_TYPE op_type) {
    switch (op_type) {
        case type::OPERATION_TYPE::OP_MULT: return std::multiplies<long long>();
        case type::OPERATION_TYPE::OP_DIV: return std::divides<long long>();
        case type::OPERATION_TYPE::OP_MOD: return std::modulus<long long>();
        case type::OPERATION_TYPE::OP_ADD: return std::plus<long long>();
        case type::OPERATION_TYPE::OP_SUB: return std::minus<long long>();
        case type::OPERATION_TYPE::OP_GREATER: return std::greater<long long>();
        case type::OPERATION_TYPE::OP_LESS: return std::less<long long>();
        case type::OPERATION_TYPE::OP_GE: return std::greater_equal<long long>();
        case type::OPERATION_TYPE::OP_LE: return std::less_equal<long long>();
        case type::OPERATION_TYPE::OP_EQUAL: return std::equal_to<long long>();
        case type::OPERATION_TYPE::OP_NE: return std::not_equal_to<long long>();
        case type::OPERATION_TYPE::OP_LOGICAL_AND: return std::logical_and<long long>();
        case type::OPERATION_TYPE::OP_LOGICAL_OR: return std::logical_or<long long>();
        default: last_arm_gen_erro="Unsupported operation type.";
    }
}

bool check_literal(std::shared_ptr<Node> const &node, std::shared_ptr<LiteralNode> &literal) {
    last_arm_gen_erro = nullptr;
    type::NODE_TYPE type = node->get_type();

    switch (type) {
        case type::NODE_TYPE::N_PARENTHESIS: {
            auto parenthesis_node = std::dynamic_pointer_cast<ParenthesisNode>(node);
            return check_literal(parenthesis_node->get_node(), literal);
        }
        
        case type::NODE_TYPE::N_LITERAL: {
            auto literal_node = std::dynamic_pointer_cast<LiteralNode>(node);
            literal = std::make_shared<LiteralNode>(*literal_node);
            return true;
        }
        
        case type::NODE_TYPE::N_OPERATION: {
            auto operation_node = std::dynamic_pointer_cast<OperationNode>(node);
            type::OPERATION_TYPE op_type = operation_node->get_op_type();
            auto parameters = operation_node->get_parameters();
            auto parent = operation_node->get_parent();

            switch (op_type) {
                case type::OPERATION_TYPE::OP_UNARY_ADD:
                case type::OPERATION_TYPE::OP_UNARY_MINUS:
                case type::OPERATION_TYPE::OP_LOGICAL_NOT: {
                    if (!check_literal(parameters[0], literal))
                        return false;

                    literal->set_parent(parent);
                    if (op_type == type::OPERATION_TYPE::OP_UNARY_MINUS) {
                        literal->set_value(-literal->get_value());
                    } else if (op_type == type::OPERATION_TYPE::OP_LOGICAL_NOT) {
                        literal->set_value(!literal->get_value());
                    }
                    return true;
                }
                
                case type::OPERATION_TYPE::OP_MULT:
                case type::OPERATION_TYPE::OP_DIV:
                case type::OPERATION_TYPE::OP_MOD:
                case type::OPERATION_TYPE::OP_ADD:
                case type::OPERATION_TYPE::OP_SUB:
                case type::OPERATION_TYPE::OP_GREATER:
                case type::OPERATION_TYPE::OP_LESS:
                case type::OPERATION_TYPE::OP_GE:
                case type::OPERATION_TYPE::OP_LE:
                case type::OPERATION_TYPE::OP_EQUAL:
                case type::OPERATION_TYPE::OP_NE:
                case type::OPERATION_TYPE::OP_LOGICAL_AND:
                case type::OPERATION_TYPE::OP_LOGICAL_OR: {
                    auto op_function = get_operation_function(op_type);
                    if (!calculation(parameters, parent, op_function, literal))
                        return false;
                    return true;
                }
                
                case type::OPERATION_TYPE::OP_ASSIGNMENT:
                    last_arm_gen_erro = "Assignments are not allowed within expressions!";
                    return false;

                default:
                    last_arm_gen_erro = "Invalid operation type!";
                    return false;
            }
        }
        
        case type::NODE_TYPE::N_VAR: {
            auto var_node = std::dynamic_pointer_cast<VarNode>(node);
            auto target_var = var_node->get_target_var();
            auto var_def = std::dynamic_pointer_cast<VarFuncNode>(target_var->get_parent());

            if (!var_def->is_const()) {
                last_arm_gen_erro = "The target variable is not marked as const!";
                return false;
            }

            auto var_types = var_def->get_var_type();
            auto it = std::find(var_types.begin(), var_types.end(), target_var);
            if (it == var_types.end()) {
                last_arm_gen_erro = "Target variable not found in the parent's var_type list!";
                return false;
            }

            auto init_value = var_def->get_var_value()[std::distance(var_types.begin(), it)];
            if (init_value->get_type() != type::NODE_TYPE::N_LITERAL) {
                last_arm_gen_erro = "The initial value of the const variable is not a literal!";
                return false;
            }

            literal = std::dynamic_pointer_cast<LiteralNode>(init_value);
            return true;
        }
        
        default:
            last_arm_gen_erro = "Unsupported node type in check_literal!";
            return false;
    }
}

bool array_gen(
    const std::vector<int> &dimensions,
    const std::shared_ptr<ArrayNode> &init_value,
    std::vector<long long> &_init_value,
    std::vector<int> &init_array_index,
    int dimensions_index
) {
    const auto &array = init_value->get_array();

    if (dimensions_index >= dimensions.size()) {
        int index = init_array_index.back();

        if (index < array.size()) {
            auto element = array[index];
            switch (element->get_type()) {
                case type::NODE_TYPE::N_LITERAL:
                    _init_value.push_back(std::dynamic_pointer_cast<LiteralNode>(element)->get_value());
					++init_array_index.back();
                    break;
                case type::NODE_TYPE::N_ARRAY:
                    _init_value.push_back(0);
                    break;
                default:
                    std::shared_ptr<LiteralNode> literal;
                    if (!check_literal(element, literal)) {
                        last_arm_gen_erro = "Array element cannot be resolved to a literal value!";
                        return false;
                    }
                    literal->set_parent(init_value);
                    _init_value.push_back(literal->get_value());
					++init_array_index.back();
                    break;
            }
        } else {
           _init_value.push_back(0);
        }
    } else {
        for (int i = 0; i < dimensions[dimensions_index]; ++i) {
            int index = init_array_index.back();
            if (dimensions_index == init_array_index.size() - 1 &&
                index < array.size() &&
                array[index]->get_type() == type::NODE_TYPE::N_ARRAY) {
                init_array_index.push_back(0);
                if (!array_gen(dimensions, std::dynamic_pointer_cast<ArrayNode>(array[index]), _init_value, init_array_index, dimensions_index + 1)) {
                    return false;
                }
                init_array_index.pop_back();
				++init_array_index.back();
            } else {
                if (!array_gen(dimensions, init_value, _init_value, init_array_index, dimensions_index + 1)) {
                    return false;
                }
            }
        }
    }
    return true;
}

bool array_gen(std::vector<int> const &dimensions, std::shared_ptr<ArrayNode> const &init_value, std::vector<std::shared_ptr<Node>> &_init_value,
		std::vector<int> &init_array_index, int dimensions_index) {
	std::vector<std::shared_ptr<Node>> const &array = init_value->get_array();
	if (dimensions_index >= dimensions.size()) {
		int index = init_array_index.back();
		if (array.size() > index) {
			if (type::NODE_TYPE::N_ARRAY != array[index]->get_type()) {
				_init_value.push_back(array[index]);
				++init_array_index.back();
				return true;
			}
		}
		std::shared_ptr<LiteralNode> zero(new LiteralNode(nullptr, type::NODE_TYPE::N_LITERAL, 0));
		_init_value.push_back(std::dynamic_pointer_cast<Node>(zero));
		return true;
	}
	for (int i = 0, end = dimensions[dimensions_index]; i < end; ++i) {
		int index = init_array_index.back();
		if (init_array_index.size() - 1 == dimensions_index && array.size() > index && type::NODE_TYPE::N_ARRAY == array[index]->get_type()) {
			init_array_index.push_back(0);
			if (!array_gen(dimensions, std::dynamic_pointer_cast<ArrayNode>(array[index]), _init_value, init_array_index, dimensions_index + 1))
				return false;
			init_array_index.pop_back();
			++init_array_index.back();
		} else {
			if (!array_gen(dimensions, init_value, _init_value, init_array_index, dimensions_index + 1))
				return false;
		}
	}
	return true;
}

bool array_gen(std::vector<int> const &dimensions, std::shared_ptr<ArrayNode> &init_value, std::vector<long long> &_init_value) {
	std::vector<int> init_array_index = { 0 };
	return array_gen(dimensions, init_value, _init_value, init_array_index, 0);
}

bool array_gen(std::vector<int> const &dimensions, std::shared_ptr<ArrayNode> &init_value, std::vector<std::shared_ptr<Node>> &_init_value) {
	std::vector<int> init_array_index = { 0 };
	return array_gen(dimensions, init_value, _init_value, init_array_index, 0);
}

bool calculate_block_var_offset(std::shared_ptr<BlockNode> &block, int &local_var_total_size) {
    const auto &children = block->get_children();

    for (const auto &child : children) {
        const auto type = child->get_type();

        switch (type) {
            case type::NODE_TYPE::N_VAR_DEF: {
                auto var_func_node = std::dynamic_pointer_cast<VarFuncNode>(child);
                const auto &var_types = var_func_node->get_var_type();

                for (const auto &var_type : var_types) {
                    const auto &dimensions = var_type->get_array();
                    long long size = 1;

                    if (!dimensions.empty()) {
                        for (const auto &dimension_node : dimensions) {
                            if (dimension_node->get_type() != type::NODE_TYPE::N_LITERAL) {
                                last_arm_gen_erro = "The array dimension must be a literal value!";
                                return false;
                            }
                            const int dimension = std::dynamic_pointer_cast<LiteralNode>(dimension_node)->get_value();
                            if (dimension < 0) {
                                last_arm_gen_erro = "Invalid dimension size!";
                                return false;
                            }
                            size *= dimension;
                        }
                    }
                    if (size < 0) {
                        last_arm_gen_erro = "The array size must be specified!";
                        return false;
                    }
                    var_func_node->add_var_offset(-(local_var_total_size += (size + 1) * 4));
                }
                break;
            }
            case type::NODE_TYPE::N_STMT: {
                auto stmt_node = std::dynamic_pointer_cast<StmtNode>(child);
                const auto &stmt = stmt_node->get_stmt();
                if (!process_statement(stmt, local_var_total_size)) {
                    return false;
                }
                break;
            }
            case type::NODE_TYPE::N_IF_ELSE:
            case type::NODE_TYPE::N_WHILE:
            case type::NODE_TYPE::N_BLOCK: {
                if (!process_statement(child, local_var_total_size)) {
                    return false;
                }
                break;
            }
            default:
                break;
        }
    }
    return true;
}

bool process_statement(const std::shared_ptr<Node> &stmt, int &local_var_total_size) {
    const auto type = stmt->get_type();
    switch (type) {
        case type::NODE_TYPE::N_IF_ELSE: {
            auto if_else_node = std::dynamic_pointer_cast<IfElseNode>(stmt);
            auto if_body = if_else_node->get_if_body();
            if (!calculate_block_var_offset(if_body, local_var_total_size)) {
                return false;
            }
            auto else_body = if_else_node->get_else_body();
            if (else_body && !calculate_block_var_offset(else_body, local_var_total_size)) {
                return false;
            }
            break;
        }
        case type::NODE_TYPE::N_WHILE: {
            auto while_node = std::dynamic_pointer_cast<WhileNode>(stmt);
            auto body = while_node->get_body();
            if (!calculate_block_var_offset(body, local_var_total_size)) {
                return false;
            }
            break;
        }
        case type::NODE_TYPE::N_BLOCK: {
            auto block_node = std::dynamic_pointer_cast<BlockNode>(stmt);
            if (!calculate_block_var_offset(block_node, local_var_total_size)) {
                return false;
            }
            break;
        }
        default:
            break;
    }
    return true;
}

bool calculate_funtion_var_offset(std::shared_ptr<FuncDefNode> &fun_def, int &local_var_total_size) {
	const auto &parameters = fun_def->get_parameters();
	for (const auto &param : parameters) {
		const auto &var_type = param->get_var_type();
		if (1 != var_type.size()) {
			last_arm_gen_erro = "Invalid function parameter format!";
			return false;
		}
		param->add_var_offset(-(local_var_total_size += 4));
	}
	auto body = fun_def->get_body();
	return calculate_block_var_offset(body, local_var_total_size);
}

std::shared_ptr<Node> get_var_type_init_value(std::shared_ptr<VarTypeNode> const &type) {
	const auto &var_def = std::dynamic_pointer_cast<VarFuncNode>(type->get_parent());
	const auto &var_type = var_def->get_var_type();
	const auto &var_init_value = var_def->get_var_value();
	for (int i = 0, end = var_type.size(); i < end; ++i) {
		if (type == var_type[i])
			return var_init_value[i];
	}
	return nullptr;
}

bool literal_vector2int_vector(std::vector<std::shared_ptr<Node>> const &literal_vector, std::vector<int> &int_vector) {
	for (const auto& literal_node : literal_vector) {
		if (type::NODE_TYPE::N_LITERAL != literal_node->get_type()) {
			last_arm_gen_erro = "The value of the vector element is not a literal!";
			return false;
		}
		auto _literal_node = std::dynamic_pointer_cast<LiteralNode>(literal_node);
		int_vector.push_back(_literal_node->get_value());
	}
	return true;
}

bool get_array_access_index_expression(std::shared_ptr<VarTypeNode> const &target_var, std::vector<std::shared_ptr<Node>> const &indexes,
    std::shared_ptr<Node> &expression, bool &partial_access) {
    std::vector<int> dimensions;
    if (!literal_vector2int_vector(target_var->get_array(), dimensions))
        return false;
    int indexes_size = indexes.size();
    int dimensions_size = dimensions.size();
    if (dimensions_size < indexes_size) {
        last_arm_gen_erro = "Error: Array index exceeds the number of dimensions!";
        return false;
    }
    partial_access = indexes_size != dimensions_size;
    std::shared_ptr<Node> index = indexes[0];
    for (int i = 1; i < dimensions_size; ++i) {
        auto dimension_node = std::make_shared<LiteralNode>(nullptr, type::NODE_TYPE::N_LITERAL, dimensions[i]);
        auto mul_node = std::make_shared<OperationNode>(nullptr, type::NODE_TYPE::N_OPERATION, type::OPERATION_TYPE::OP_MULT, index, dimension_node);
        dimension_node->set_parent(mul_node);
        index->set_parent(mul_node);

        auto add_node = std::make_shared<OperationNode>(
            nullptr, type::NODE_TYPE::N_OPERATION, type::OPERATION_TYPE::OP_ADD, mul_node,
            (i < indexes_size) ? indexes[i] : std::make_shared<LiteralNode>(nullptr, type::NODE_TYPE::N_LITERAL, 0)
        );
        mul_node->set_parent(add_node);
        if (i < indexes_size)
            indexes[i]->set_parent(add_node);

        index = add_node;
    }
    expression = index;
    return true;
}

bool process_var_def(std::shared_ptr<Node> const &node, std::ostringstream &os) {
    std::shared_ptr<VarFuncNode> _node = std::dynamic_pointer_cast<VarFuncNode>(node);
    if (_node->is_global()) {
        os << "	.data" << std::endl;
        const auto &var_type = _node->get_var_type();
        const auto &var_init_value = _node->get_var_value();
        for (int i = 0; i < var_type.size(); ++i) {
            if (!process_global_var(var_type[i], var_init_value[i], os)) {
                return false;
            }
        }
        os << "	.text" << std::endl;
        for (int i = 0; i < var_type.size(); ++i) {
            const auto &type = var_type[i];
            os << "." << type->get_name() << "_addr:" << std::endl;
            os << "	.word	" << type->get_name() << std::endl;
        }
        os << std::endl;
    } else {
        if (!process_local_var(_node, os)) {
            return false;
        }
    }
    return true;
}

bool process_global_var(const std::shared_ptr<VarTypeNode> &type, const std::shared_ptr<Node> &init_value, std::ostringstream &os) {
    const auto &dimensions = type->get_array();
	std::vector<int>_dimensions;
	for (const auto &it : dimensions) {
		if (it->get_type() != type::NODE_TYPE::N_LITERAL) {
			last_arm_gen_erro = "The dimension is not a literal!";
			return false;
		}
		std::shared_ptr<LiteralNode> _it = std::dynamic_pointer_cast<LiteralNode>(it);
		_dimensions.push_back(_it->get_value());
	}
	if(dimensions.size()){
		if(init_value){
			auto _init_value = std::dynamic_pointer_cast<ArrayNode>(init_value);
			std::vector<long long> __init_value;
			_init_value->get_array();
			os << type->get_name() << ":" << std::endl;
			if(!array_gen(_dimensions,_init_value,__init_value)){
				return false;
			}
			for (std::vector<long long>::iterator it = __init_value.begin(), end = __init_value.end(); it != end; ++it) {
				os << "	.word	" << (*it) << std::endl;
			}
		}
		else{
			long long size = 1;
			for (const auto &it : dimensions) {
				int dimension = std::dynamic_pointer_cast<LiteralNode>(it)->get_value();
				if (dimension < 0){
					size = -1;
				}
				size *= dimension;
			}
			if (size=-1) {
				last_arm_gen_erro = "The array size must be specified!";
				return false;
			}
			os << type->get_name() << ":" << std::endl;
			while(size--){
				os << "	.word	0" << std::endl;
			}
			
		}
	}
	return true;
}

bool process_local_var(std::shared_ptr<VarFuncNode> const &node, std::ostringstream &os) {
    const auto &var_type = node->get_var_type();
    const auto &var_init_value = node->get_var_value();
    for (int i = 0; i < var_type.size(); ++i) {
        if (!process_local_var_item(var_type[i], var_init_value[i], os)) {
            return false;
        }
    }
    return true;
}

bool process_local_var_item(const std::shared_ptr<VarTypeNode> &type, const std::shared_ptr<Node> &_init_value, std::ostringstream &os) {
    const auto &dimensions = type->get_array();
    std::vector<int> _dimensions;
    for (const auto &it : dimensions) {
        if (it->get_type() != type::NODE_TYPE::N_LITERAL) {
            last_arm_gen_erro = "The dimension is not a literal!";
            return false;
        }
        _dimensions.push_back(std::dynamic_pointer_cast<LiteralNode>(it)->get_value());
    }

    auto store_value = [&os](int reg, int offset) {
        if (is_valid_immediate_value(offset)) {
            os << "	str	r" << reg << ", [fp, #" << offset << "]" << std::endl;
        } else {
            int temp_register = reg_allocate(os);
            os << "	ldr	r" << temp_register << ", =" << offset << std::endl;
            os << "	str	r" << reg << ", [fp, r" << temp_register << "]" << std::endl;
            reg_deallocate(temp_register, os);
        }
    };

    if (dimensions.size()) {
        int offset;
        if (!get_target_var_offset(type, offset))
            return false;
        int temp_register = reg_allocate(os);
        int immediate_value = offset + 4;
        if (is_valid_immediate_value(immediate_value)) {
            os << "	add	r" << temp_register << ", fp, #" << immediate_value << std::endl;
        } else {
            os << "	ldr	r" << temp_register << ", =" << immediate_value << std::endl;
            os << "	add	r" << temp_register << ", fp, r" << temp_register << std::endl;
        }
        os << "	str	r" << temp_register << ", [r" << temp_register << ", #-4]" << std::endl;
        reg_deallocate(temp_register, os);
        if (_init_value) {
            if (_init_value->get_type() != type::NODE_TYPE::N_ARRAY) {
                last_arm_gen_erro = "Init array must use array literal!";
                return false;
            }
            auto init_value = std::dynamic_pointer_cast<ArrayNode>(_init_value);
            std::vector<std::shared_ptr<Node>> __init_value;
            if (!array_gen(_dimensions, init_value, __init_value))
                return false;
            int index = 1;
            for (const auto &it : __init_value) {
                if (!arm_gen(it, os))
                    return false;
                
                immediate_value = offset + 4 * index;
                store_value(10, immediate_value);
                ++index;
            }
        }
    } else {
        if (_init_value) {
            if (!arm_gen(_init_value, os))
                return false;

            int offset;
            if (!get_target_var_offset(type, offset))
                return false;

            store_value(10, offset);
        }
    }
    return true;
}

bool process_func_def(std::shared_ptr<Node> const &node, std::ostringstream &os) {
    auto _node = std::dynamic_pointer_cast<FuncDefNode>(node);
    int local_var_total_size = 0;
    if (!calculate_funtion_var_offset(_node, local_var_total_size)) {
        return false;
    }
    long long return_label_id = compiler::next_global_id();
    _node->set_return_label_id(return_label_id);
    _node->set_local_var_total_size(local_var_total_size);
    os << _node->get_name() << ":" << std::endl;
    os << "	push	{fp, lr}" << std::endl;
    os << "	add	fp, sp, #0" << std::endl;
	process_func_prologue(local_var_total_size, os);
    const auto &parameters = _node->get_parameters();
    if (!process_func_parameters(parameters, os)) {
        return false;
    }
    auto body = _node->get_body();
    if (!arm_gen(std::dynamic_pointer_cast<Node>(body), os)) {
        return false;
    }
    os << ".L" << return_label_id << ":" << std::endl;
    os << "	sub	sp, fp, #0" << std::endl;
    os << "	pop	{fp, pc}" << std::endl;
    os << "	.ltorg" << std::endl;
    os << std::endl;

    return true;
}

bool process_func_prologue(int local_var_total_size, std::ostringstream &os) {
    int immediate_value = local_var_total_size;
	if (is_valid_immediate_value(immediate_value)) {
		os << "	sub	sp, sp, #" << immediate_value << std::endl;
	} else {
		int temp_register = reg_allocate(os);
		os << "	ldr	r" << temp_register << ", =" << immediate_value << std::endl;
		os << "	sub	sp, sp, r" << temp_register << std::endl;
		reg_deallocate(temp_register, os);
	}
	return true;
}

bool process_func_parameters(const std::vector<std::shared_ptr<VarFuncNode>> &parameters, std::ostringstream &os) {
    int parameters_size = parameters.size();

    auto store_value = [&os](int reg, int offset) {
        if (is_valid_immediate_value(offset)) {
            os << "	str	r" << reg << ", [fp, #" << offset << "]" << std::endl;
        } else {
            int temp_register = reg_allocate(os);
            os << "	ldr	r" << temp_register << ", =" << offset << std::endl;
            os << "	str	r" << reg << ", [fp, r" << temp_register << "]" << std::endl;
            reg_deallocate(temp_register, os);
        }
    };

    auto load_value = [&os](int reg, int offset) {
        if (is_valid_immediate_value(offset)) {
            os << "	ldr	r" << reg << ", [fp, #" << offset << "]" << std::endl;
        } else {
            int temp_register = reg_allocate(os);
            os << "	ldr	r" << temp_register << ", =" << offset << std::endl;
            os << "	ldr	r" << reg << ", [fp, r" << temp_register << "]" << std::endl;
            reg_deallocate(temp_register, os);
        }
    };
    
    for (int i = 0, end = std::min(parameters_size, 4); i < end; ++i) {
        const auto &var_def = parameters[i];
        int immediate_value = var_def->get_off_set()[0];
        store_value(i, immediate_value);
    }
    for (int i = 0; i < (parameters_size > 4 ? parameters_size - 4 : 0); ++i) {
        const auto &var_def = parameters[4 + i];
        int stack_offset = 8 + 4 * i;

        load_value(10, stack_offset);

        int immediate_value = var_def->get_off_set()[0];
        store_value(10, immediate_value);
    }
    return true;
}

bool process_var(std::shared_ptr<Node> const &node, std::ostringstream &os) {
    auto var_node = std::dynamic_pointer_cast<VarNode>(node);
    const auto &target_var = var_node->get_target_var();
    bool is_global = std::dynamic_pointer_cast<VarFuncNode>(target_var->get_parent())->is_global();
    if (is_global) {
        os << "	ldr	r10, ." << var_node->get_name() << "_addr" << std::endl;
        if (target_var->get_array().empty() || get_var_type_init_value(target_var)) {
            os << "	ldr	r10, [r10]" << std::endl;
        }
    } else {
        int offset;
        if (!get_target_var_offset(target_var, offset)) {
            return false;
        }
        int immediate_value = offset;
        if (is_valid_immediate_value(immediate_value)) {
            os << "	ldr	r10, [fp, #" << immediate_value << "]" << std::endl;
        } else {
            int temp_register = reg_allocate(os);
            os << "	ldr	r" << temp_register << ", =" << immediate_value << std::endl;
            os << "	ldr	r10, [fp, r" << temp_register << "]" << std::endl;
            reg_deallocate(temp_register, os);
        }
    }
    return true;
}

bool process_func(std::shared_ptr<Node> const &node, std::ostringstream &os) {
    auto func_node = std::dynamic_pointer_cast<FuncNode>(node);
    const auto &parameters = func_node->get_parameters();
    int param_count = parameters.size();
    os << "	push	{r0-r9}" << std::endl;
    for (int i = 0; i < std::min(param_count, 4); ++i) {
        if (!arm_gen(parameters[i], os)) {
            return false;
        }
        os << "	mov	r" << i << ", r10" << std::endl;
    }
    for (int i = 4; i < param_count; ++i) {
        if (!arm_gen(parameters[i], os)) {
            return false;
        }
        os << "	push	{r10}" << std::endl;
    }
    os << "	bl	" << func_node->get_name() << std::endl;
    for (int i = param_count - 1; i >= 4; --i) {
        os << "	pop	{r10}" << std::endl;
    }
    os << "	mov	r10, r0" << std::endl;
    os << "	pop	{r0-r9}" << std::endl;
    return true;
}

bool process_array_index(std::shared_ptr<Node> const &node, std::ostringstream &os) {
    auto _node = std::dynamic_pointer_cast<ArrayAccessNode>(node);
    const auto &target = _node->get_target();
    if (type::NODE_TYPE::N_VAR != target->get_type()) {
        last_arm_gen_erro = "ArrayIndexNode's target must be variable!";
        return false;
    }
    const auto &indexes = _node->get_indexes();
    auto var_node = std::dynamic_pointer_cast<VarNode>(target);
    const auto &target_var = var_node->get_target_var();
    std::shared_ptr<Node> expression;
    bool partial_access;
    if (!get_array_access_index_expression(target_var, indexes, expression, partial_access)) {
        return false;
    }
    expression->set_parent(node);
    if (!arm_gen(expression, os)) {
        return false;
    }
    bool is_global = std::dynamic_pointer_cast<VarFuncNode>(target_var->get_parent())->is_global();
    int temp_register = reg_allocate(os);
    if (is_global) {
        os << "	ldr	r" << temp_register << ", ." << var_node->get_name() << "_addr" << std::endl;
        if (!get_var_type_init_value(target_var)) {
            os << "	ldr	r" << temp_register << ", [r" << temp_register << "]" << std::endl;
        }
    } else {
        int offset;
        if (!get_target_var_offset(target_var, offset)) {
            return false;
        }
        int immediate_value = offset;
        if (is_valid_immediate_value(immediate_value)) {
            os << "	ldr	r" << temp_register << ", [fp, #" << immediate_value << "]" << std::endl;
        } else {
            os << "	ldr	r" << temp_register << ", =" << immediate_value << std::endl;
            os << "	ldr	r" << temp_register << ", [fp, r" << temp_register << "]" << std::endl;
        }
    }
    os << "	add	r10, r" << temp_register << ", r10, LSL #2" << std::endl;
    if (!partial_access) {
        os << "	ldr	r10, [r10]" << std::endl;
    }
    reg_deallocate(temp_register, os);
    return true;
}

bool arm_gen(std::shared_ptr<Node> const &node, std::ostringstream &os){
    last_arm_gen_erro = nullptr;

    type::NODE_TYPE type = node->get_type();

	switch (type) {
	case type::NODE_TYPE::N_VAR_DEF: {
		return process_var_def(node, os);
	} break;
	case type::NODE_TYPE::N_FUNC_DEF: {
		return process_func_def(node, os);
	} break;
	case type::NODE_TYPE::N_VAR: {
		return process_var(node, os);
	} break;
	case type::NODE_TYPE::N_FUNC: {
		return process_func(node,os);
	} break;
	case type::NODE_TYPE::N_STMT: {
		auto _node = std::dynamic_pointer_cast<StmtNode>(node);
		if (!arm_gen(std::dynamic_pointer_cast<Node>(_node->get_stmt()), os))
			return false;
	} break;
	case type::NODE_TYPE::N_PARENTHESIS: {
		auto _node = std::dynamic_pointer_cast<ParenthesisNode>(node);
		if (!arm_gen(std::dynamic_pointer_cast<Node>(_node->get_node()), os))
			return false;
	} break;
	case type::NODE_TYPE::N_LITERAL: {
		const auto &_node = std::dynamic_pointer_cast<LiteralNode>(node);
		int immediate_value = _node->get_value();
		if (is_valid_immediate_value(immediate_value)) {
			os << "	mov	r10, #" << immediate_value << std::endl;
		} else {
			os << "	ldr	r10, =" << immediate_value << std::endl;
		}
	} break;
	case type::NODE_TYPE::N_ARRAY_INDEX: {
		return process_array_index(node,os);
	} break;
	case type::NODE_TYPE::N_OPERATION: {
        std::shared_ptr<OperationNode> _node = std::dynamic_pointer_cast<OperationNode>(node);
        std::vector<std::shared_ptr<Node> >& parameters = _node->get_parameters();

        int temp_register = reg_allocate(os);
        type::OPERATION_TYPE operator_type = _node->get_op_type();

        switch (operator_type) {
            // 直接处理二元操作符的共通逻辑
            case type::OPERATION_TYPE::OP_ADD:
            case type::OPERATION_TYPE::OP_SUB:
            case type::OPERATION_TYPE::OP_MULT: {
                if (!arm_gen(parameters[0], os)){
                    last_arm_gen_erro="Left operand generation failed.";
                    return false;
                }
                os << "  mov r" << temp_register << ", r10" << std::endl;
                if (!arm_gen(parameters[1], os)) {
                    last_arm_gen_erro="Right operand generation failed.";
                    return false;
                }
                switch (operator_type) {
                    case type::OPERATION_TYPE::OP_ADD: os << "  add r10, r" << temp_register << ", r10" << std::endl; break;
                    case type::OPERATION_TYPE::OP_SUB: os << "  sub r10, r" << temp_register << ", r10" << std::endl; break;
                    case type::OPERATION_TYPE::OP_MULT: os << "  mul r10, r" << temp_register << ", r10" << std::endl; break;
                }
                break;
            }
        case type::OPERATION_TYPE::OP_UNARY_ADD:
            if (!arm_gen(parameters[0], os))
                return false;
            break;
        case type::OPERATION_TYPE::OP_UNARY_MINUS:
            if (!arm_gen(parameters[0], os))
                return false;
            os << "	rsb	r10, r10, #0" << std::endl;
            break;
        case type::OPERATION_TYPE::OP_LOGICAL_NOT:
            if (!arm_gen(parameters[0], os))
                return false;
            os << "	cmp	r10, #0" << std::endl;
            os << "	moveq	r10, #1" << std::endl;
            os << "	movne	r10, #0" << std::endl;
            break;
   
        case type::OPERATION_TYPE::OP_DIV:
        {
            os << "	push	{r0-r3}" << std::endl;
            if (!arm_gen(parameters[0], os))
                return false;
            os << "	mov	r0, r10" << std::endl;
            if (!arm_gen(parameters[1], os))
                return false;
            os << "	mov	r1, r10" << std::endl;
            os << "	bl	__aeabi_idiv" << std::endl;
            os << "	mov	r10, r0" << std::endl;
            os << "	pop	{r0-r3}" << std::endl;
        }
        break;
        case type::OPERATION_TYPE::OP_MOD: {
                os << "  push {r0-r3}" << std::endl; // 保存寄存器
                if (!arm_gen(parameters[0], os)){
                    last_arm_gen_erro="Operand generation failed.";
                    return false;
                }
                os << "  mov r0, r10" << std::endl;
                if (!arm_gen(parameters[1], os)){
                    last_arm_gen_erro="Operand generation failed.";
                    return false;
                }
                os << "  mov r1, r10" << std::endl;
                os << "  bl __aeabi_idiv" << (operator_type == type::OPERATION_TYPE::OP_MOD ? "mod" : "") << std::endl;
                os << "  mov r10, " << (operator_type == type::OPERATION_TYPE::OP_DIV ? "r0" : "r1") << std::endl;
                os << "  pop {r0-r3}" << std::endl; // 恢复寄存器
                break;
            }

        case type::OPERATION_TYPE::OP_GREATER:
            if (!arm_gen(parameters[0], os))
                return false;
            os << "	mov	r" << temp_register << ", r10" << std::endl;
            if (!arm_gen(parameters[1], os))
                return false;
            os << "	cmp	r" << temp_register << ", r10" << std::endl;
            os << "	movgt	r10, #1" << std::endl;
            os << "	movle	r10, #0" << std::endl;
            break;
        case type::OPERATION_TYPE::OP_LESS:
            if (!arm_gen(parameters[0], os))
                return false;
            os << "	mov	r" << temp_register << ", r10" << std::endl;
            if (!arm_gen(parameters[1], os))
                return false;
            os << "	cmp	r" << temp_register << ", r10" << std::endl;
            os << "	movlt	r10, #1" << std::endl;
            os << "	movge	r10, #0" << std::endl;
            break;
        case type::OPERATION_TYPE::OP_GE:
            if (!arm_gen(parameters[0], os))
                return false;
            os << "	mov	r" << temp_register << ", r10" << std::endl;
            if (!arm_gen(parameters[1], os))
                return false;
            os << "	cmp	r" << temp_register << ", r10" << std::endl;
            os << "	movge	r10, #1" << std::endl;
            os << "	movlt	r10, #0" << std::endl;
            break;
        case type::OPERATION_TYPE::OP_LE:
            if (!arm_gen(parameters[0], os))
                return false;
            os << "	mov	r" << temp_register << ", r10" << std::endl;
            if (!arm_gen(parameters[1], os))
                return false;
            os << "	cmp	r" << temp_register << ", r10" << std::endl;
            os << "	movle	r10, #1" << std::endl;
            os << "	movgt	r10, #0" << std::endl;
            break;
        case type::OPERATION_TYPE::OP_EQUAL:
            if (!arm_gen(parameters[0], os))
                return false;
            os << "	mov	r" << temp_register << ", r10" << std::endl;
            if (!arm_gen(parameters[1], os))
                return false;
            os << "	cmp	r" << temp_register << ", r10" << std::endl;
            os << "	moveq	r10, #1" << std::endl;
            os << "	movne	r10, #0" << std::endl;
            break;
        case type::OPERATION_TYPE::OP_NE:
            if (!arm_gen(parameters[0], os))
                return false;
            os << "	mov	r" << temp_register << ", r10" << std::endl;
            if (!arm_gen(parameters[1], os))
                return false;
            os << "	cmp	r" << temp_register << ", r10" << std::endl;
            os << "	movne	r10, #1" << std::endl;
            os << "	moveq	r10, #0" << std::endl;
            break;
        case type::OPERATION_TYPE::OP_LOGICAL_AND: {
            int end_zero_label_id = compiler::next_global_id();
            int end_label_id = compiler::next_global_id();
            if (!arm_gen(parameters[0], os))
                return false;
            os << "	cmp	r10, #0" << std::endl;
            os << "	beq	.L" << end_zero_label_id << std::endl;
            if (!arm_gen(parameters[1], os))
                return false;
            os << "	cmp	r10, #0" << std::endl;
            os << "	beq	.L" << end_zero_label_id << std::endl;

            os << "	mov	r10, #1" << std::endl;
            os << "	b	.L" << end_label_id << std::endl;
            os << ".L" << end_zero_label_id << ":" << std::endl;
            os << "	mov	r10, #0" << std::endl;
            os << ".L" << end_label_id << ":" << std::endl;
        }break;
        case type::OPERATION_TYPE::OP_LOGICAL_OR: {
            int end_one_label_id = compiler::next_global_id();
            int end_label_id = compiler::next_global_id();
            if (!arm_gen(parameters[0], os))
                return false;
            os << "	cmp	r10, #0" << std::endl;
            os << "	bne	.L" << end_one_label_id << std::endl;
            if (!arm_gen(parameters[1], os))
                return false;
            os << "	cmp	r10, #0" << std::endl;
            os << "	bne	.L" << end_one_label_id << std::endl;
            os << "	mov	r10, #0" << std::endl;
            os << "	b	.L" << end_label_id << std::endl;
            os << ".L" << end_one_label_id << ":" << std::endl;
            os << "	mov	r10, #1" << std::endl;
            os << ".L" << end_label_id << ":" << std::endl;
        }break;
        case type::OPERATION_TYPE::OP_ASSIGNMENT: {
            std::shared_ptr<Node> const& node = parameters[0];
            type::NODE_TYPE type = node->get_type();
            if (type::NODE_TYPE::N_VAR != type && type::NODE_TYPE::N_ARRAY_INDEX != type) {
                last_arm_gen_erro = "Assignment operator first parameter must be lvalue!!";
                return false;
            }
            if (!arm_gen(parameters[1], os))
                return false;
            if (type::NODE_TYPE::N_VAR == type) {
                std::shared_ptr<VarNode> _node = std::dynamic_pointer_cast<VarNode>(node);
                const std::shared_ptr<VarTypeNode>& target_var = _node->get_target_var();
                if (std::dynamic_pointer_cast<VarFuncNode>(target_var->get_parent())->is_global()) {
                    os << "	ldr	r" << temp_register << ", ." << _node->get_name() << "_addr" << std::endl;
                    os << "	str	r10, [r" << temp_register << "]" << std::endl;
                }
                else {
                    int offset;
                    if (!get_target_var_offset(target_var, offset))
                        return false;
                    int immediate_value = offset;
                    if (is_valid_immediate_value(immediate_value)) {
                        os << "	str	r10, [fp, #" << immediate_value << "]" << std::endl;
                    }
                    else {
                        os << "	ldr	r" << temp_register << ", =" << immediate_value << std::endl;
                        os << "	str	r10, [fp, r" << temp_register << "]" << std::endl;
                    }
                }
            }
            else {
                std::shared_ptr<ArrayAccessNode> _node = std::dynamic_pointer_cast<ArrayAccessNode>(node);
                std::shared_ptr<Node> const& target = _node->get_target();
                if (type::NODE_TYPE::N_VAR != target->get_type()) {
                    last_arm_gen_erro = "ArrayAccessNode's target must be variable!!";
                    return false;
                }
                std::vector<std::shared_ptr<Node>> const& indexes = _node->get_indexes();
                std::shared_ptr<VarNode> var_node = std::dynamic_pointer_cast<VarNode>(target);
                const std::shared_ptr<VarTypeNode>& target_var = var_node->get_target_var();

                std::shared_ptr<Node> expression;
                bool partitial_access;
                if (!get_array_access_index_expression(target_var, indexes, expression, partitial_access))
                    return false;
                if (partitial_access) {
                    last_arm_gen_erro = "Assignment operation lvalue cannot be an array!!";
                    return false;
                }
                expression->set_parent(node);
                os << "	push	{r10}" << std::endl;
                if (!arm_gen(expression, os))
                    return false;
                if (std::dynamic_pointer_cast<VarFuncNode>(target_var->get_parent())->is_global()) {
                    os << "	ldr	r" << temp_register << ", ." << var_node->get_name() << "_addr" << std::endl;
                    if (!get_var_type_init_value(target_var)) {
                        os << "	ldr	r" << temp_register << ", [r" << temp_register << "]" << std::endl;
                    }
                    os << "	add	r" << temp_register << ", r" << temp_register << ", r10, LSL #2" << std::endl;
                    os << "	pop	{r10}" << std::endl;
                    os << "	str	r10, [r" << temp_register << "]" << std::endl;
                }
                else {
                    int offset;
                    if (!get_target_var_offset(target_var, offset))
                        return false;
                    int immediate_value = offset;
                    if (is_valid_immediate_value(immediate_value)) {
                        os << "	ldr	r" << temp_register << ", [fp, #" << immediate_value << "]" << std::endl;
                    }
                    else {
                        os << "	ldr	r" << temp_register << ", =" << immediate_value << std::endl;
                        os << "	ldr	r" << temp_register << ", [fp, r" << temp_register << "]" << std::endl;
                    }
                    os << "	add	r" << temp_register << ", r" << temp_register << ", r10, LSL #2" << std::endl;
                    os << "	pop	{r10}" << std::endl;
                    os << "	str	r10, [r" << temp_register << "]" << std::endl;
                }
            }
        }break;
        default:
            last_arm_gen_erro="not supported operation type!";
            return false;
        }
        reg_deallocate(temp_register, os);
        break;
	}break;


	case type::NODE_TYPE::N_IF_ELSE: {
            auto _node = std::dynamic_pointer_cast<IfElseNode>(node);
            const auto else_label_id = compiler::next_global_id();
            const auto end_label_id = compiler::next_global_id();

            if (!arm_gen(_node->get_condition(), os))
                return false;
            os << "   cmp r10, #0" << std::endl;
            os << "   beq .L" << else_label_id << std::endl;

            if (!arm_gen(_node->get_if_body(), os))
                return false;
            os << "   b .L" << end_label_id << std::endl;

            os << ".L" << else_label_id << ":" << std::endl;
            if (_node->get_else_body() && !arm_gen(_node->get_else_body(), os))
                return false;

            os << ".L" << end_label_id << ":" << std::endl;
            break;
        }

        case type::NODE_TYPE::N_WHILE: {
            auto _node = std::dynamic_pointer_cast<WhileNode>(node);
            const auto continue_label_id = compiler::next_global_id();
            const auto break_label_id = compiler::next_global_id();

            _node->set_continue_label_id(continue_label_id);
            _node->set_break_label_id(break_label_id);

            os << ".L" << continue_label_id << ":" << std::endl;
            if (!arm_gen(_node->get_condition(), os))
                return false;
            os << "   cmp r10, #0" << std::endl;
            os << "   beq .L" << break_label_id << std::endl;

            if (!arm_gen(_node->get_body(), os))
                return false;
            os << "   b .L" << continue_label_id << std::endl;

            os << ".L" << break_label_id << ":" << std::endl;
            break;
        }

        case type::NODE_TYPE::N_CONTINUE: {
            auto _node = std::dynamic_pointer_cast<ContinueNode>(node);
            os << "   b .L" << _node->get_target()->get_continue_label_id() << std::endl;
            break;
        }

        case type::NODE_TYPE::N_BREAK: {
            auto _node = std::dynamic_pointer_cast<BreakNode>(node);
            os << "   b .L" << _node->get_target()->get_break_label_id() << std::endl;
            break;
        }

        case type::NODE_TYPE::N_RETURN: {
            auto _node = std::dynamic_pointer_cast<ReturnNode>(node);
            if (_node->get_return_value()) {
                if (!arm_gen(_node->get_return_value(), os))
                    return false;
                os << "   mov r0, r10" << std::endl;
            }
            os << "   b .L" << _node->get_target()->get_return_label_id() << std::endl;
            break;
        }

        case type::NODE_TYPE::N_BLOCK: {
            auto _node = std::dynamic_pointer_cast<BlockNode>(node);
            if (!arm_gen_array(_node->get_children(), os))
                return false;
            break;
        }
    case type::NODE_TYPE::N_PIN: {
        //直接跳过
	} break;
	case type::NODE_TYPE::N_FLOAT_LITERAL:
	case type::NODE_TYPE::N_TYPE:
	case type::NODE_TYPE::N_VAR_TYPE:
	case type::NODE_TYPE::N_DEF:
	case type::NODE_TYPE::N_REF:
	case type::NODE_TYPE::N_ARRAY:
	default: {
		last_arm_gen_erro = "Not supported node type!!";
		return false;
	}
	}
	return true;
}

bool arm_gen_head(std::shared_ptr<Node> const &node, std::ostringstream &os) {
    // ControlFlowGraph cfg;
    // build_cfg(node, cfg);

    // SSAConverter ssa(cfg);
    // ssa.convert();

    // replace_phi_nodes(cfg);

	os << "	.global	main" << std::endl;
	return arm_gen(node, os);
}
