#include "./parser.h"
#include "../util/node.h"
#include "../util/type.h"
#include "fmt/core.h"
#include "fmt/format.h"
#include <cassert>
#include <iterator>
#include <memory>
#include <stdexcept>
#include <string>

namespace compiler::frontend {
namespace type = compiler::type;

auto Parser::gen_block_child_node(vec_shared_token::const_iterator &curr,
                                  vec_shared_token::const_iterator &end,
                                  bool &is_end) -> std::shared_ptr<Node> {
  is_end = false;
  if (curr == end) {
    is_end = true;
    return nullptr;
  }
  auto type = (*curr)->get_type();
  switch (type) {
  case type::TOKEN_TYPE::T_SYMBOL: {
    auto name = (*curr)->get_name();
    if (name != "(") {
      if (name == ";") { // 空表达式
        ++curr;
        return std::make_shared<StmtNode>(nullptr, type::NODE_TYPE::N_STMT,
                                          nullptr);
      }
      if (name == "{") {
        ++curr;
        auto block_node = gen_block_node(curr, end);
        if (!block_node) {
          return nullptr;
        }
        if (!expect(curr, end, "}")) {
          return nullptr;
        }
        return block_node;
      } else {
        is_end = true;
        return nullptr;
      }
    }
  }
  case type::TOKEN_TYPE::T_IDENTIFIER:
  case type::TOKEN_TYPE::T_OPERATOR:
  case type::TOKEN_TYPE::T_DEC_DIGIT:
  case type::TOKEN_TYPE::T_HEX_DIGIT:
  case type::TOKEN_TYPE::T_OCT_DIGIT: {
    auto node = gen_expr_node(curr, end);
    if (!node) {
      return nullptr;
    }
    // gen_expr必不能为INVALID
    assert(node->get_type() != type::NODE_TYPE::N_INVALID);
    auto stmt_node =
        std::make_shared<StmtNode>(nullptr, type::NODE_TYPE::N_STMT, node);
    node->set_parent(stmt_node);
    if (!expect(curr, end, ";")) {
      return nullptr;
    }
    return stmt_node;
  }
  case type::TOKEN_TYPE::T_CONST: {
    ++curr;
    auto var_def_node = gen_multi_variable_definition_node(curr, end);
    if (!var_def_node) {
      return nullptr;
    }
    var_def_node->set_const(true);
    return var_def_node;
  }
  case type::TOKEN_TYPE::T_INT:
  case type::TOKEN_TYPE::T_FLOAT:
  case type::TOKEN_TYPE::T_VOID: {
    // 获取类型定义
    auto type_node = gen_type_node(curr, end);
    if (!type_node) {
      return nullptr;
    }
    auto type = (*curr)->get_type();
    if (type != type::TOKEN_TYPE::T_IDENTIFIER) {
      return nullptr;
    }
    // 获取变量名
    auto name = (*curr)->get_name();
    ++curr;
    // 如果是函数
    if (expect(curr, end, "(")) {
      auto func_def_node = std::make_shared<FuncDefNode>();
      func_def_node->set_return_type(type_node);
      func_def_node->set_name(name).set_type(type::NODE_TYPE::N_FUNC_DEF);
      type_node->set_parent(func_def_node);
      if (!expect(curr, end, ")")) {
        do {
          auto var_def_node = gen_single_variable_definition_node(curr, end);
          if (!var_def_node) {
            return nullptr;
          }
          func_def_node->add_parameter(var_def_node);
          var_def_node->set_parent(func_def_node);
        } while (expect(curr, end, ","));
        if (!expect(curr, end, ")")) {
          return nullptr;
        }
      } // 没有参数的情况
      else {
      }
      // 函数体
      if (!expect(curr, end, ";")) {
        if (!expect(curr, end, "{")) {
          return nullptr;
        }
        auto block_node = gen_block_node(curr, end);
        if (!block_node) {
          return nullptr;
        }
        func_def_node->set_body(block_node);
        block_node->set_parent(func_def_node);
        if (!expect(curr, end, "}")) {
          return nullptr;
        }
      }
      return func_def_node;
    } // 不是函数的情况
    else {
      --curr;
      // VarFuncNode就是定义一个变量，该变量可能为函数
      // FIXME: 传入一个正确的name
      // 2024-07-22 11:44:49 ""改为了(*curr)->get_name()
      auto var_def_node =
          std::make_shared<VarFuncNode>(nullptr, type::NODE_TYPE::N_VAR_DEF,
                                        (*curr)->get_name(), type_node, false);
      while (curr != end) {
        // 这里传出的Node的name部队
        auto var_type = gen_var_type_node(curr, end);
        if (!var_type) {
          return nullptr;
        }
        std::shared_ptr<Node> var_init_value;
        if (expect(curr, end, "=")) {
          var_init_value = gen_expr_node(curr, end);
          if (!var_init_value) {
            return nullptr;
          }
        }
        var_def_node->add_var(var_type, var_init_value);
        var_type->set_parent(var_def_node);
        if (var_init_value) {
          var_init_value->set_parent(var_def_node);
        }
        if (expect(curr, end, ",")) {
          continue;
        }
        if (expect(curr, end, ";")) {
          break;
        }
        return nullptr;
      }
      return var_def_node;
    }
  }
  case type::TOKEN_TYPE::T_IF: {
    auto if_else_node = gen_if_else_node(curr, end);
    if (!if_else_node) {
      return nullptr;
    }
    return if_else_node;
  }
  case type::TOKEN_TYPE::T_WHILE: {
    ++curr;
    if (!expect(curr, end, "(")) {
      return nullptr;
    }
    auto condition = gen_expr_node(curr, end);
    if (!condition) {
      return nullptr;
    }
    if (!expect(curr, end, ")")) {
      return nullptr;
    }
    auto body = gen_block_or_single_line_block(curr, end);
    if (!body) {
      return nullptr;
    }
    auto while_node = std::make_shared<WhileNode>(
        nullptr, type::NODE_TYPE::N_WHILE, condition, body);
    condition->set_parent(while_node);
    body->set_parent(while_node);
    return while_node;
  }
  case type::TOKEN_TYPE::T_CONTINUE: {
    ++curr;
    if (!expect(curr, end, ";")) {
      return nullptr;
    }
    return std::make_shared<ContinueNode>(nullptr, type::NODE_TYPE::N_CONTINUE);
  }
  case type::TOKEN_TYPE::T_BREAK: {
    ++curr;
    if (!expect(curr, end, ";")) {
      return nullptr;
    }
    return std::make_shared<BreakNode>(nullptr, type::NODE_TYPE::N_BREAK);
  }
  case type::TOKEN_TYPE::T_RETURN: {
    ++curr;
    std::shared_ptr<Node> return_value;
    if (!expect(curr, end, ";")) {
      return_value = gen_expr_node(curr, end);
      if (!return_value) {
        return nullptr;
      }
      if (!expect(curr, end, ";")) {
        return nullptr;
      }
    }
    auto return_node = std::make_shared<ReturnNode>(
        nullptr, type::NODE_TYPE::N_RETURN, return_value);
    if (return_value) {
      return_value->set_parent(return_node);
    }
    return return_node;
  }
  case type::TOKEN_TYPE::T_ELSE:
  case type::TOKEN_TYPE::T_INVALID:
  default: {
    return nullptr;
  }
  }
  return nullptr;
}
auto Parser::gen_block_node(vec_shared_token::const_iterator &current,
                            vec_shared_token::const_iterator &end)
    -> std::shared_ptr<BlockNode> {
  auto block_node =
      std::make_shared<BlockNode>(nullptr, type::NODE_TYPE::N_BLOCK);
  while (current != end) {
    bool is_end = false;
    auto child_node = gen_block_child_node(current, end, is_end);
    if (is_end) {
      goto while_end;
    }
    if (!child_node) {
      return nullptr;
    }
    block_node->add_children(child_node);
    child_node->set_parent(block_node);
  }
while_end:
  return block_node;
}
auto Parser::parse() -> std::shared_ptr<BlockNode> {
  vec_shared_token::const_iterator current = tokens.begin(), end = tokens.end();
  auto root_block = gen_block_node(current, end);
  if (!root_block) {
    return nullptr;
  }
  if (current != end) {
    return nullptr;
  }
  return root_block;
}
auto Parser::jsonify(const std::shared_ptr<Node> &node) -> std::string {
  auto type = node->get_type();
  fmt::memory_buffer buffer;
  switch (type) {
  case type::NODE_TYPE::N_TYPE: {
    auto type_node = std::dynamic_pointer_cast<TypeNode>(node);
    fmt::format_to(std::back_inserter(buffer),
                   R"("type": "{}","variable_type":"{}", "dimensions": {} )",
                   type, type_node->get_variable_type(),
                   array_to_json(type_node->get_array()));
  } break;
  case type::NODE_TYPE::N_VAR_TYPE: {
    auto var_type_node = std::dynamic_pointer_cast<VarTypeNode>(node);
    fmt::format_to(std::back_inserter(buffer),
                   R"("type": "{}", "name": "{}", "dimensions": {})",
                   var_type_node->get_type(),
                   var_type_node->get_name() == "" ? "name"
                                                   : var_type_node->get_name(),
                   array_to_json(var_type_node->get_array()));
  } break;
  case type::NODE_TYPE::N_DEF: {
    auto def_node = std::dynamic_pointer_cast<DefNode>(node);
    fmt::format_to(std::back_inserter(buffer), R"("type": "{}", "name": "{}")",
                   type::node_type_to_string(type), def_node->get_name());
  } break;
  case type::NODE_TYPE::N_VAR_DEF: {
    auto var_def_node = std::dynamic_pointer_cast<VarFuncNode>(node);
    fmt::format_to(
        std::back_inserter(buffer),
        R"("type": "{}","name":"{}","variable_type":{},"var_init_value":{},"is_const":"{}")",
        type::node_type_to_string(type), var_def_node->get_name(),
        array_to_json(var_def_node->get_var_type()),
        array_to_json(var_def_node->get_var_value()), var_def_node->is_const());
    ;
  } break;
  case type::NODE_TYPE::N_FUNC_DEF: {
    auto func_def_node = std::dynamic_pointer_cast<FuncDefNode>(node);
    fmt::format_to(
        std::back_inserter(buffer),
        R"("type":"{}","name":"{}","return_type":{},"parameters":{},"body":{})",
        type, func_def_node->get_name(),
        jsonify(func_def_node->get_return_type()),
        array_to_json(func_def_node->get_parameters()),
        jsonify(func_def_node->get_body()));
  } break;
  case type::NODE_TYPE::N_REF: {
    auto ref_node = std::dynamic_pointer_cast<RefNode>(node);
    // 偏特化，不再手动处理NODE_TYPE的打印
    fmt::format_to(std::back_inserter(buffer), R"("type": "{}", "name": "{}")",
                   type, ref_node->get_name());
  } break;
  case type::NODE_TYPE::N_VAR: {
    auto var_node = std::dynamic_pointer_cast<VarNode>(node);
    fmt::format_to(std::back_inserter(buffer), R"("type":"{}","name":"{}")",
                   type, var_node->get_name());

  } break;
  case type::NODE_TYPE::N_FUNC: {
    auto func_node = std::dynamic_pointer_cast<FuncNode>(node);
    fmt::format_to(std::back_inserter(buffer),
                   R"("type":"{}","name":"{}","parameters":{})", type,
                   func_node->get_name(),
                   array_to_json(func_node->get_parameters()));
  } break;
  case type::NODE_TYPE::N_STMT: {
    auto stmt_node = std::dynamic_pointer_cast<StmtNode>(node);
    auto sub_stmt = stmt_node->get_stmt();
    fmt::format_to(std::back_inserter(buffer), R"("type":"{}")", type);
    if (sub_stmt != nullptr) {
      fmt::format_to(std::back_inserter(buffer), R"( "stmt": {})",
                     jsonify(sub_stmt));
    }
  } break;
  case type::NODE_TYPE::N_PARENTHESIS: {
    auto parenthesis_node = std::dynamic_pointer_cast<ParenthesisNode>(node);
    fmt::format_to(std::back_inserter(buffer), R"("type":"{}","node":{})", type,
                   jsonify(parenthesis_node->get_node()));
  } break;
    // FIXME: 这个位置get_value会崩溃
    // 可能是因为指针转换失败
  case type::NODE_TYPE::N_FLOAT_LITERAL:
  case type::NODE_TYPE::N_LITERAL: {
    // auto literal_node = std::dynamic_pointer_cast<LiteralNode>(node);
    if (auto float_literal_node =
            std::dynamic_pointer_cast<FloatLiteralNode>(node)) {
      fmt::format_to(std::back_inserter(buffer), R"("type":"{}")", type);
    } else if (auto literal_node =
                   std::dynamic_pointer_cast<LiteralNode>(node)) {
      fmt::format_to(std::back_inserter(buffer), R"("type": "{}")", type);
    } else {
      throw std::runtime_error("Can not execute dynamic_pointer_cast.");
    }
  } break;
  case type::NODE_TYPE::N_ARRAY: {
    auto array_node = std::dynamic_pointer_cast<ArrayNode>(node);
    fmt::format_to(std::back_inserter(buffer), R"("type":"{}","array":{})",
                   type, array_to_json(array_node->get_array()));
  } break;
  case type::NODE_TYPE::N_ARRAY_INDEX: {
    auto array_index_node = std::dynamic_pointer_cast<ArrayAccessNode>(node);
    fmt::format_to(std::back_inserter(buffer),
                   R"("type":"{}","indexes":{},"target":{})", type,
                   array_to_json(array_index_node->get_indexes()),
                   jsonify(array_index_node->get_target()));
  } break;
  case type::NODE_TYPE::N_OPERATION: {
    auto op_node = std::dynamic_pointer_cast<OperationNode>(node);
    fmt::format_to(std::back_inserter(buffer),
                   R"("type":"{}","operator_type":"{}","parameters":{})", type,
                   type::op_type_to_string(op_node->get_op_type()),
                   array_to_json(op_node->get_parameters()));
  } break;
  case type::NODE_TYPE::N_IF_ELSE: {
    auto if_else_node = std::dynamic_pointer_cast<IfElseNode>(node);
    fmt::format_to(std::back_inserter(buffer),
                   R"("type":"{}","condition":{},"if_body":{},"else_body":{})",
                   type, jsonify(if_else_node->get_condition()),
                   jsonify(if_else_node->get_if_body()),
                   if_else_node->get_else_body() != nullptr
                       ? jsonify(if_else_node->get_else_body())
                       : "");
  } break;
  case type::NODE_TYPE::N_WHILE: {
    auto while_node = std::dynamic_pointer_cast<WhileNode>(node);
    fmt::format_to(std::back_inserter(buffer),
                   R"("type":"{}","condition":{},"body":{})", type,
                   jsonify(while_node->get_condition()),
                   jsonify(while_node->get_body()));
  } break;
  case type::NODE_TYPE::N_CONTINUE: {
    auto continue_node = std::dynamic_pointer_cast<ContinueNode>(node);
    fmt::format_to(std::back_inserter(buffer), R"("type":"{}")", type);
  } break;
  case type::NODE_TYPE::N_BREAK: {
    auto break_node = std::dynamic_pointer_cast<BreakNode>(node);
    fmt::format_to(std::back_inserter(buffer), R"("type":"{}")", type);

  } break;
  case type::NODE_TYPE::N_RETURN: {
    auto return_node = std::dynamic_pointer_cast<ReturnNode>(node);
    fmt::format_to(std::back_inserter(buffer),
                   R"("type":"{}","return_value":{})", type,
                   jsonify(return_node->get_return_value()));
  } break;
  case type::NODE_TYPE::N_BLOCK: {
    auto block_node = std::dynamic_pointer_cast<BlockNode>(node);
    fmt::format_to(std::back_inserter(buffer), R"("type":"{}","children":{})",
                   type, array_to_json(block_node->get_children()));
  } break;
  default: {
    throw std::runtime_error("Unsupported syntax");
  }
  }
  return fmt::format("{{{}}}", to_string(buffer));
}
} // namespace compiler::frontend
