#pragma once
#include "../util/node.h"
#include "../util/type.h"
#include "./tokens.h"
#include "fmt/core.h"
#include "fmt/format.h"
#include <cstddef>
#include <exception>
#include <iostream>
#include <iterator>
#include <memory>
#include <stack>
#include <string>
#include <vector>

using vec_shared_token =
    std::vector<std::shared_ptr<compiler::frontend::Token>>;
namespace compiler::frontend {
using namespace compiler::node;
namespace type = compiler::type;
class Token;
class Parser {
private:
  std::vector<std::shared_ptr<Token>> tokens;

public:
  Parser() = default;
  explicit Parser(std::vector<std::shared_ptr<Token>> &tokens)
      : tokens(tokens) {}
  auto jsonify(const std::shared_ptr<Node> &node) -> std::string;
  auto parse() -> std::shared_ptr<BlockNode>;

  auto gen_block_child_node(vec_shared_token::const_iterator &current,
                            vec_shared_token::const_iterator &end,
                            bool &reached_end) -> std::shared_ptr<Node>;
  auto gen_block_node(vec_shared_token::const_iterator &current,
                      vec_shared_token::const_iterator &end)
      -> std::shared_ptr<BlockNode>;

private:
  auto expect(vec_shared_token::const_iterator &current,
              vec_shared_token::const_iterator &end,
              const char *const str) -> bool {
    return current != end && (*current)->get_name() == str && (++current, 1);
  }
  template <typename T>
  auto
  array_to_json(const std::vector<std::shared_ptr<T>> &nodes) -> std::string {
    fmt::memory_buffer buffer;
    if (nodes.empty()) {
      return "[]";
    }
    for (int i = 0; i < nodes.size(); i++) {
      const auto &node = nodes[i];

      if (node != nullptr) {
        if (std::is_same_v<T, Node>) {
          fmt::format_to(std::back_inserter(buffer), "{}", jsonify(node));
        } else {
          fmt::format_to(std::back_inserter(buffer), "{}",
                         jsonify(std::dynamic_pointer_cast<Node>(node)));
        }
        if (i != nodes.size() - 1) {
          buffer.push_back(',');
        }
      }
    }
    return fmt::format("[{}]", fmt::to_string(buffer));
  }
  // 生成操作符的节点
  auto gen_top_operator_operation_node(
      std::stack<type::OPERATION_TYPE> &operator_stack,
      std::stack<std::shared_ptr<Node>> &node_stack) -> bool {
    if (operator_stack.empty()) {
      return false;
    }
    auto top_operatpr = operator_stack.top();
    operator_stack.pop();
    auto expr_node = std::make_shared<OperationNode>();
    expr_node->set_type(type::NODE_TYPE::N_OPERATION);
    for (std::size_t i = 0; i < sysy_operator_param_number(top_operatpr); i++) {
      if (node_stack.empty()) {
        return false;
      }
      auto &node = node_stack.top();
      node_stack.pop();
      expr_node->add_parameter(node);
      node->set_parent(std::dynamic_pointer_cast<Node>(expr_node));
    }
    expr_node->reverse_parameters().set_op_type(top_operatpr);
    node_stack.push(std::dynamic_pointer_cast<Node>(expr_node));
    return true;
  }
  // 生成表达式（用一个节点表示）
  auto gen_expr_node(vec_shared_token::const_iterator &current,
                     vec_shared_token::const_iterator &end)
      -> std::shared_ptr<Node> {
    bool prev_is_operator = true;
    std::stack<type::OPERATION_TYPE> op_stack;
    std::stack<std::shared_ptr<Node>> node_stack;
    for (auto type = (*current)->get_type();; type = (*current)->get_type()) {
      switch (type) {
      case type::TOKEN_TYPE::T_OPERATOR: {
        if (prev_is_operator) {
          auto o_type =
              type::str_to_op_type((*current)->get_name().c_str(), true);
          if (o_type == type::OPERATION_TYPE::OP_INVALID) {
            return nullptr;
          }
          op_stack.push(o_type);
        } else {
          auto o_type =
              type::str_to_op_type((*current)->get_name().c_str(), false);
          if (o_type == type::OPERATION_TYPE::OP_INVALID) {
            return nullptr;
          }
          if (type::sysy_operator_assoc(o_type) == type::ASSOCIATIVITY::LEFT) {
            // op非空并且栈内的运算符的优先级更高，就把当前的op放入栈（运算时不需要考虑优先级顺序）
            while (!op_stack.empty() &&
                   type::sysy_operator_prec(op_stack.top()) >=
                       type::sysy_operator_prec(o_type)) {
              gen_top_operator_operation_node(op_stack, node_stack);
            }
            op_stack.push(o_type);
          } else {
            while (!op_stack.empty() &&
                   type::sysy_operator_prec(op_stack.top()) >
                       type::sysy_operator_prec(o_type)) {
              gen_top_operator_operation_node(op_stack, node_stack);
            }
            op_stack.push(o_type);
          }
        }
        ++current;
        prev_is_operator = true;
      } break;
      case type::TOKEN_TYPE::T_INT:
      case type::TOKEN_TYPE::T_DEC_DIGIT:
      case type::TOKEN_TYPE::T_HEX_DIGIT:
      case type::TOKEN_TYPE::T_OCT_DIGIT: {
        auto literal_node = std::make_shared<LiteralNode>();
        literal_node->set_type(type::NODE_TYPE::N_LITERAL);
        try {
          literal_node->set_value((*current)->get_name().c_str());
        } catch (std::exception &e) {
          std::cerr << e.what();
          return nullptr;
        }
        node_stack.push(std::dynamic_pointer_cast<Node>(literal_node));
        ++current;
        prev_is_operator = false;
      } break;
      case type::TOKEN_TYPE::T_FLOAT: {
        auto float_literal_node = std::make_shared<FloatLiteralNode>();
        float_literal_node->set_type(type::NODE_TYPE::N_FLOAT_LITERAL);
        try {
          float_literal_node->set_value((*current)->get_name().c_str());
        } catch (std::exception &e) {
          std::cerr << e.what();
          return nullptr;
        }
        node_stack.push(std::dynamic_pointer_cast<Node>(float_literal_node));
        ++current;
        prev_is_operator = false;
      } break;
      case type::TOKEN_TYPE::T_IDENTIFIER: {
        const auto &name = (*current)->get_name();
        // 带有括号为函数，否则为变量名
        if (expect(current, end, "(")) {
          auto func_node = std::make_shared<FuncNode>(
              nullptr, type::NODE_TYPE::N_FUNC, name.c_str());
          node_stack.push(std::dynamic_pointer_cast<Node>(func_node));
          if (!expect(current, end, ")")) {
            do {
              auto node = gen_expr_node(current, end);
              if (!node) {
                return nullptr;
              }
              node->set_parent(std::dynamic_pointer_cast<Node>(func_node));
              func_node->add_parameter(node);
            } while (expect(current, end, ","));
            if (!expect(current, end, ")")) {
              return nullptr;
            }
          } else {
            // 无参数的函数
          }
        } else {
          auto var_node = std::make_shared<VarNode>(
              nullptr, type::NODE_TYPE::N_VAR, name.c_str());
          node_stack.push(std::dynamic_pointer_cast<Node>(var_node));
        }
        // TODO: 这个位置是后来添加的，可能有潜在的bug
        if ((*current)->get_name() == name) {
          ++current;
        }
        prev_is_operator = false;
      } break;
      case type::TOKEN_TYPE::T_SYMBOL: {
        // 形如 a*(b+c)
        if ((*current)->get_name() == "(") {
          ++current;
          auto node = gen_expr_node(current, end);
          if (!node) {
            return nullptr;
          }
          auto parenthesis_node = std::make_shared<ParenthesisNode>(
              nullptr, type::NODE_TYPE::N_PARENTHESIS, node);
          node->set_parent(std::dynamic_pointer_cast<Node>(parenthesis_node));
          node_stack.push(std::dynamic_pointer_cast<Node>(parenthesis_node));
          if (!expect(current, end, ")")) {
            return nullptr;
          }
          prev_is_operator = false;
          break;
        }
        // 形如{2,3,4}
        if ((*current)->get_name() == "{") {
          ++current;
          auto array_node =
              std::make_shared<ArrayNode>(nullptr, type::NODE_TYPE::N_ARRAY);
          node_stack.push(std::dynamic_pointer_cast<Node>(array_node));
          // 获取每一个元素
          if (!expect(current, end, "}")) {
            do {
              auto node = gen_expr_node(current, end);
              if (!node) {
                return nullptr;
              }
              node->set_parent(std::dynamic_pointer_cast<Node>(array_node));
              array_node->add_array(node);
            } while (expect(current, end, ","));
            if (!expect(current, end, "}")) {
              return nullptr;
            }

          } else {
            // 初始化列表没有元素
          }
          prev_is_operator = false;
          break;
        }
        // 访问数组
        if ((*current)->get_name() == "[") {
          ++current;
          if (node_stack.empty()) {
            return nullptr;
          }
          auto array_access_node = std::make_shared<ArrayAccessNode>();
          auto &top_node = node_stack.top();
          if (top_node->get_type() != type::NODE_TYPE::N_ARRAY_INDEX) {
            array_access_node = std::make_shared<ArrayAccessNode>(
                nullptr, type::NODE_TYPE::N_ARRAY_INDEX, top_node);
            top_node->set_parent(
                std::dynamic_pointer_cast<Node>(array_access_node));
            // 需要更新当前的node, 所以要抛出栈顶
            node_stack.pop();
            node_stack.push(std::dynamic_pointer_cast<Node>(array_access_node));
          } else {
            // 栈顶的node是访问数组的，直接获取
            array_access_node =
                std::dynamic_pointer_cast<ArrayAccessNode>(top_node);
          }
          // 现在为[]内的表达式，如果空则返回（应当有报错）
          auto node = gen_expr_node(current, end);
          if (!node) {
            return nullptr;
          }
          node->set_parent(std::dynamic_pointer_cast<Node>(array_access_node));
          array_access_node->add_index(node);
          if (!expect(current, end, "]")) {
            return nullptr;
          }
          prev_is_operator = false;
          break;
        }
      }
      default:
        goto for_end;
      }
      if (current == end) {
        break;
      }
    }
  for_end:
    while (op_stack.size()) {
      if (!gen_top_operator_operation_node(op_stack, node_stack)) {
        return nullptr;
      }
    }
    if (node_stack.size() != 1) {
      return nullptr;
    }
    auto node = node_stack.top();
    node_stack.pop();
    return node;
  }
  auto gen_type_node(vec_shared_token::const_iterator &current,
                     vec_shared_token::const_iterator &end)
      -> std::shared_ptr<TypeNode> {
    auto type = (*current)->get_type();
    ++current;
    auto type_node = std::make_shared<TypeNode>(
        nullptr, type::NODE_TYPE::N_TYPE, type::token_type_to_val_type(type));
    while (expect(current, end, "[")) {
      if (expect(current, end, "]")) {
        auto literal_node = std::make_shared<LiteralNode>(
            nullptr, type::NODE_TYPE::N_LITERAL, -1);
        if (type == type::TOKEN_TYPE::T_FLOAT) {
          literal_node = std::make_shared<FloatLiteralNode>(
              nullptr, type::NODE_TYPE::N_FLOAT_LITERAL, -1.0);
        }
        auto node = std::dynamic_pointer_cast<Node>(literal_node);
        type_node->add_array(node);
        node->set_parent(std::dynamic_pointer_cast<Node>(type_node));

      } else {
        auto node = gen_expr_node(current, end);
        type_node->add_array(node);
        node->set_parent(std::dynamic_pointer_cast<Node>(type_node));
        if (expect(current, end, "]")) {
          return nullptr;
        }
      }
    }
    return type_node;
  }
  // 生成变量类型如int[][], float[]
  // FIXME: 这个位置生成VarTypeNode时，array会出现奇怪的变量
  //
  auto gen_var_type_node(vec_shared_token::const_iterator &current,
                         vec_shared_token::const_iterator &end)
      -> std::shared_ptr<VarTypeNode> {
    auto type = (*current)->get_type();
    auto name = (*current)->get_name();
    auto var_type_node = std::make_shared<VarTypeNode>(
        nullptr, type::NODE_TYPE::N_VAR_TYPE, name.c_str());
    ++current;
    while (expect(current, end, "[")) {
      if (expect(current, end, "]")) {
        auto literal_node = std::make_shared<LiteralNode>(
            nullptr, type::NODE_TYPE::N_LITERAL, -1);
        if (type == type::TOKEN_TYPE::T_FLOAT) {
          literal_node = std::make_shared<FloatLiteralNode>(
              nullptr, type::NODE_TYPE::N_FLOAT_LITERAL, -1.0);
        }
        auto node = std::dynamic_pointer_cast<Node>(literal_node);
        var_type_node->add_array(node);
        node->set_parent(var_type_node);
      } else {
        auto node = gen_expr_node(current, end);
        var_type_node->add_array(node);
        node->set_parent(var_type_node);
        if (!expect(current, end, "]")) {
          return nullptr;
        }
      }
    }
    return var_type_node;
  }
  // 单个参数的函数
  auto gen_single_variable_definition_node(
      vec_shared_token::const_iterator &current,
      vec_shared_token::const_iterator &end) -> std::shared_ptr<VarFuncNode> {
    auto type_node = gen_type_node(current, end);
    if (!type_node) {
      return nullptr;
    }
    auto var_def_node = std::make_shared<VarFuncNode>(
        nullptr, type::NODE_TYPE::N_VAR_DEF, "", type_node, false);
    auto var_type = gen_var_type_node(current, end);
    if (!var_type) {
      return nullptr;
    }
    std::shared_ptr<Node> var_init_value;
    if (expect(current, end, "=")) {
      var_init_value = gen_expr_node(current, end);
      if (!var_init_value) {
        return nullptr;
      }
    }
    var_def_node->add_var(var_type, var_init_value);
    var_type->set_parent(std::dynamic_pointer_cast<Node>(var_def_node));
    if (var_init_value) {
      var_init_value->set_parent(std::dynamic_pointer_cast<Node>(var_def_node));
    }
    return var_def_node;
  }
  auto gen_multi_variable_definition_node(
      vec_shared_token::const_iterator &current,
      vec_shared_token::const_iterator &end) -> std::shared_ptr<VarFuncNode> {
    auto type_node = gen_type_node(current, end);
    if (!type_node) {
      return nullptr;
    }
    auto var_def_node = std::make_shared<VarFuncNode>(
        nullptr, type::NODE_TYPE::N_VAR_DEF, "", type_node, false);
    while (current != end) {
      auto var_type = gen_var_type_node(current, end);
      if (!var_type) {
        return nullptr;
      }
      std::shared_ptr<Node> var_init_value;
      if (expect(current, end, "=")) {
        var_init_value = gen_expr_node(current, end);
        if (!var_init_value) {
          return nullptr;
        }
      }
      var_def_node->add_var(var_type, var_init_value);
      var_type->set_parent(std::dynamic_pointer_cast<Node>(var_def_node));
      if (var_init_value) {
        var_init_value->set_parent(
            std::dynamic_pointer_cast<Node>(var_def_node));
      }
      // 定义还没结束
      if (expect(current, end, ",")) {
        continue;
      }
      if (expect(current, end, ";")) {
        break; // 定义结束
      }
      return nullptr;
    }
    return var_def_node;
  }
  auto gen_block_or_single_line_block(vec_shared_token::const_iterator &current,
                                      vec_shared_token::const_iterator &end)
      -> std::shared_ptr<BlockNode> {
    std::shared_ptr<BlockNode> block_node;
    if (expect(current, end, "{")) {
      block_node = gen_block_node(current, end);
      if (!block_node) {
        return nullptr;
      }
      if (!expect(current, end, "}")) {
        return nullptr;
      }
    } else {
      bool is_end = false;
      auto child_node = gen_block_child_node(current, end, is_end);
      if (is_end) {
        return nullptr;
      }
      if (!child_node) {
        return nullptr;
      }
      block_node = std::make_shared<BlockNode>(
          nullptr, type::NODE_TYPE::N_BLOCK, child_node);
      child_node->set_parent(block_node);
    }
    return block_node;
  }
  auto gen_if_else_node(vec_shared_token::const_iterator &curr,
                        vec_shared_token::const_iterator &end)
      -> std::shared_ptr<IfElseNode> {
    if (!expect(curr, end, "if")) {
      return nullptr;
    }
    if (!expect(curr, end, "(")) {
      return nullptr;
    }
    auto condition_node = gen_expr_node(curr, end);
    if (!condition_node) {
      return nullptr;
    }
    if (!expect(curr, end, ")")) {
      return nullptr;
    }
    auto if_body = gen_block_or_single_line_block(curr, end);
    if (!if_body) {
      return nullptr;
    }
    auto else_body = std::make_shared<BlockNode>();
    if (!expect(curr, end, "else")) {
    } // 没有else语句
    else {
      else_body = gen_block_or_single_line_block(curr, end);
      if (!else_body) {
        return nullptr;
      }
    }
    auto if_else_node =
        std::make_shared<IfElseNode>(nullptr, type::NODE_TYPE::N_IF_ELSE,
                                     condition_node, if_body, else_body);
    condition_node->set_parent(if_else_node);
    if_body->set_parent(if_else_node);
    if (else_body) {
      else_body->set_parent(if_else_node);
    }
    return if_else_node;
  }
};
} // namespace compiler::frontend
