#pragma once
#include "../util/type.h"
#include "./tokens.h"
#include <array>
#include <cctype>
#include <cstddef>
#include <cstdio>
#include <cstring>
#include <istream>
#include <iterator>
#include <memory>
#include <string>
#include <string_view>
#include <vector>
namespace compiler::frontend {
// 接受一个std::istream，结果为std::vector<std::shared_ptr<Token>>
// std::istream input在此处被消耗
class Tokenize {
public:
  bool is_float_literal = false;

private:
  std::string last_tokenize_error_msg;
  std::string content;
  std::vector<std::shared_ptr<Token>> tokens;

public:
  Tokenize() : last_tokenize_error_msg(), tokens() {};
  Tokenize(const std::unique_ptr<std::istream> &input)
      : last_tokenize_error_msg(), tokens() {
    content.assign(std::istreambuf_iterator<char>(*input),
                   std::istreambuf_iterator<char>());
  }

  void setup(const std::unique_ptr<std::istream> &input) {
    content.assign(std::istreambuf_iterator<char>(*input),
                   std::istreambuf_iterator<char>());
  }
  void set_last_error_msg(std::string err) {
    this->last_tokenize_error_msg = std::move(err);
  }
  [[nodiscard]] auto get_last_error_msg() const -> const std::string & {
    return last_tokenize_error_msg;
  }
  static auto singleton() -> Tokenize & {
    static Tokenize single;

    return single;
  }

  auto
  input_file_to_json() -> std::tuple<std::vector<std::shared_ptr<Token>>, bool>;
  auto dump_tokens() -> std::string;

public:
private:
  auto is_identifier_start(const char *const ptr) -> bool {
    return isalpha(*ptr) || *ptr == '_';
  }
  auto is_start_with(const char *const s1, const char *const s2) -> bool {
    return !strncmp(s1, s2, strlen(s2));
  }
  auto is_identifier_start_letter(char ch) -> bool {
    return isalpha(ch) || ch == '_';
  }
  auto is_identifier_letter(char ch) -> bool {
    return isalpha(ch) || isdigit(ch) || ch == '_';
  }
  auto new_identifier_token(const char *&current) -> std::shared_ptr<Token> {
    const char *start = current;
    while (is_identifier_letter(*(++current))) {
      ;
    }
    const char *end = current;
    return std::make_shared<Token>(start, end - start,
                                   std::string(start, end).c_str());
  }
  // 遇到浮点数时会无法移动current
  auto new_literal_token(const char *&current) -> std::shared_ptr<Token> {
    const char *start = current;
    // 十进制
    // TODO: 2024-07-17:只考虑十进制浮点数
    if ('0' != *current) {
      while (isdigit(*(++current))) {
        ;
      }

      if (*current == '.') {
        is_float_literal = true;
        while (isdigit(*(++current))) {
          ;
        }
        const char *end = current;
        current = end;
        return std::make_shared<Token>(start, end - start,
                                       std::string(start, end).c_str());
        // 这个位置不可以返回，否则会失去关键字的映射
      } else {
        const char *end = current;
        return std::make_shared<Token>(start, end - start,
                                       std::string(start, end).c_str());
      }
    }
    // 0开头
    ++current;
    // 八进制
    if ('x' != *current && 'X' != *current) {
      if (strchr("01234567", *current))
        while (strchr("01234567", *(++current))) {
          ;
        }
      const char *end = current;
      return std::make_shared<Token>(start, end - start,
                                     std::string(start, end).c_str());
    }
    // 十六进制
    {
      while (strchr("0123456789abcdefABCDEF", *(++current))) {
        ;
      }
      const char *end = current;
      return std::make_shared<Token>(start, end - start,
                                     std::string(start, end).c_str());
    }
  }
  auto is_multi_letter_symbol(const char *current) -> bool {
    for (int i = 0; i < type::sysy_multi_letter_symbol_len; i++) {
      if (is_start_with(current, type::sysy_multi_letter_symbol.at(i).data())) {
        return true;
      }
    }
    return false;
  }
  auto new_multi_letter_symbol(const char *&current) -> std::shared_ptr<Token> {
    const char *start = current;
    for (int i = 0; i < type::sysy_multi_letter_symbol_len; i++) {
      if (is_start_with(current, type::sysy_multi_letter_symbol.at(i).data())) {
        current += type::sysy_multi_letter_symbol[i].size();
        const char *end = current;
        return std::make_shared<Token>(start, end - start,
                                       std::string(start, end).c_str());
      }
    }
    // 无法到达
    return nullptr;
  }
  auto is_single_letter_symbol(const char *current) -> bool {
    for (int i = 0; i < type::sysy_single_letter_symbol_len; i++) {
      if (is_start_with(current,
                        type::sysy_single_letter_symbol.at(i).data())) {
        return true;
      }
    }
    return false;
  }
  auto
  new_single_letter_symbol(const char *&current) -> std::shared_ptr<Token> {
    const char *start = current;
    for (int i = 0; i < type::sysy_single_letter_symbol_len; i++) {
      if (is_start_with(current,
                        type::sysy_single_letter_symbol.at(i).data())) {
        current += type::sysy_single_letter_symbol.at(i).size();
        const char *end = current;
        return std::make_shared<Token>(start, end - start,
                                       std::string(start, end).c_str());
      }
    }
    // 无法到达
    return nullptr;
  }
  template <size_t len>
  auto is_included_in_array(const std::array<std::string_view, len> &array,
                            const char *const str) -> bool {
    for (int i = 0; i < len; i++) {
      if (!strcmp(str, array[i].data())) {
        return true;
      }
    }
    return false;
  }
  auto set_token_type(std::vector<std::shared_ptr<Token>> &tokens) -> bool {
    for (auto &token : tokens) {
      auto name = token->get_name().c_str();
      if (is_included_in_array(type::sysy_operator, name)) {
        token->set_type(type::TOKEN_TYPE::T_OPERATOR);
        continue;
      }
      if (is_included_in_array(type::sysy_delimeter, name)) {
        token->set_type(type::TOKEN_TYPE::T_SYMBOL);
        continue;
      }
      if (isdigit(*name)) {
        if ('0' != *name) {
          if (is_float_literal)
            token->set_type(type::TOKEN_TYPE::T_FLOAT);
          else
            token->set_type(type::TOKEN_TYPE::T_DEC_DIGIT);
        } else if ('x' == *(name + 1) || 'X' == *(name + 1)) {
          token->set_type(type::TOKEN_TYPE::T_HEX_DIGIT);
        } else {
          token->set_type(type::TOKEN_TYPE::T_OCT_DIGIT);
        }
        continue;
      }
      type::TOKEN_TYPE type = type::str_to_token_type(name);
      token->set_type(type);
      continue;
      return false;
    }
    return true;
  }
};
} // namespace compiler::frontend
