#include "node.h"
#include "type.h"
#include <algorithm>
#include <cerrno>
#include <climits>
#include <cstdint>
#include <ctime>
#include <iostream>
#include <memory>
#include <stdexcept>
#include <string>
#include <string_view>
#include <vector>
namespace compiler::node {

// Node
Node::Node() : parent(nullptr), type(type::NODE_TYPE::N_INVALID) {}
Node::Node(std::shared_ptr<Node> const &parent, type::NODE_TYPE type)
    : parent(parent), type(type) {}
auto Node::get_parent() const -> const std::shared_ptr<Node> & {
  return parent;
}
auto Node::set_parent(const std::shared_ptr<Node> &parent) -> Node & {
  this->parent = parent;
  return *this;
}
auto Node::get_type() const -> type::NODE_TYPE { return type; }
auto Node::set_type(type::NODE_TYPE type) -> Node & {
  this->type = type;
  return *this;
}
// TypeNode
TypeNode::TypeNode()
    : Node(), variable_type(type::VALUE_TYPE::V_INVALID),
      multi_dimension_array() {}
auto TypeNode::get_array() -> std::vector<std::shared_ptr<Node>> & {
  return multi_dimension_array;
}
auto TypeNode::set_array(const std::vector<std::shared_ptr<Node>> &array)
    -> TypeNode & {
  this->multi_dimension_array = array;
  return *this;
}
auto TypeNode::add_array(std::shared_ptr<Node> const &row) -> TypeNode & {
  this->multi_dimension_array.push_back(row);
  return *this;
}
auto TypeNode::get_variable_type() const -> type::VALUE_TYPE {
  return variable_type;
}
auto TypeNode::set_variable_type(type::VALUE_TYPE type) -> TypeNode & {
  this->variable_type = type;
  return *this;
}
// VarTypeNode
VarTypeNode::VarTypeNode() : Node(), name(), arrays() {}
auto VarTypeNode::get_array() -> std::vector<std::shared_ptr<Node>> & {
  return arrays;
}
auto VarTypeNode::set_array(const std::vector<std::shared_ptr<Node>> &arrays)
    -> VarTypeNode & {
  this->arrays = arrays;
  return *this;
}
auto VarTypeNode::add_array(std::shared_ptr<Node> const &row) -> VarTypeNode & {
  this->arrays.push_back(row);
  return *this;
}
auto VarTypeNode::set_name(const std::string &name) -> VarTypeNode & {
  this->name = name;
  return *this;
}
// DefNode
DefNode::DefNode() : Node(), name() {}
DefNode::DefNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE type,
                 std::string name)
    : Node(parent, type), name(std::move(name)) {}
auto DefNode::get_name() const -> const std::string & { return name; }
auto DefNode::set_name(const std::string &name) -> DefNode & {
  this->name = name;
  return *this;
}
// VarFuncNode
VarFuncNode::VarFuncNode()
    : DefNode(), variable_type(nullptr), is_const_(false), var_type(),
      var_values(), block_id(-1), is_global_(false), offset() {}
VarFuncNode::VarFuncNode(std::shared_ptr<Node> const &parent,
                         type::NODE_TYPE type, std::string name,
                         std::shared_ptr<TypeNode> const &variable_type,
                         bool is_const)
    : DefNode(parent, type, name), variable_type(variable_type),
      is_const_(is_const), var_type(), var_values(), block_id(-1),
      is_global_(false), offset() {}
auto VarFuncNode::is_const() const -> bool { return is_const_; }
auto VarFuncNode::set_const(bool id) -> VarFuncNode & {
  this->is_const_ = id;
  return *this;
}
auto VarFuncNode::get_variable_type() -> std::shared_ptr<TypeNode> & {
  return variable_type;
}
auto VarFuncNode::set_variable_type(
    const std::shared_ptr<TypeNode> &variable_type) -> VarFuncNode & {
  this->variable_type = variable_type;
  return *this;
}
auto VarFuncNode::get_var_value() -> std::vector<std::shared_ptr<Node>> & {
  return var_values;
}
auto VarFuncNode::set_var_value(
    const std::vector<std::shared_ptr<Node>> &values) -> VarFuncNode & {
  this->var_values = values;
  return *this;
}
auto VarFuncNode::get_var_type()
    -> std::vector<std::shared_ptr<VarTypeNode>> & {
  return var_type;
}
auto VarFuncNode::set_var_type(const std::vector<std::shared_ptr<VarTypeNode>>
                                   &var_type) -> VarFuncNode & {
  this->var_type = var_type;
  return *this;
}
auto VarFuncNode::add_var(std::shared_ptr<VarTypeNode> var_type,
                          std::shared_ptr<Node> var_value) -> VarFuncNode & {
  this->var_type.push_back(var_type);
  this->var_values.push_back(var_value);
  return *this;
}
auto VarFuncNode::get_block_id() const -> int { return block_id; }
auto VarFuncNode::set_block_id(int id) -> VarFuncNode & {
  this->block_id = id;
  return *this;
}
auto VarFuncNode::is_global() const -> bool { return is_global_; }
auto VarFuncNode::set_global(bool global_) -> VarFuncNode & {
  this->is_global_ = global_;
  return *this;
}
auto VarFuncNode::get_off_set() const -> const std::vector<int> & {
  return offset;
}
auto VarFuncNode::set_off_set(const std::vector<int> &offset) -> VarFuncNode & {
  this->offset = offset;
  return *this;
}
auto VarFuncNode::add_var_offset(const int offset) -> VarFuncNode & {
  this->offset.push_back(offset);
  return *this;
}
// FuncDefNode
FuncDefNode::FuncDefNode()
    : DefNode(), return_type(nullptr), body(nullptr), parameters(),
      block_id(-1), return_label_id(-1), locak_var_total_size(-1) {}
auto FuncDefNode::get_return_type() -> const std::shared_ptr<TypeNode> & {
  return return_type;
}
auto FuncDefNode::set_return_type(const std::shared_ptr<TypeNode> &type)
    -> FuncDefNode & {
  this->return_type = type;
  return *this;
}
auto FuncDefNode::get_body() const -> const std::shared_ptr<BlockNode> & {
  return body;
}
auto FuncDefNode::set_body(const std::shared_ptr<BlockNode> &body)
    -> FuncDefNode & {
  this->body = body;
  return *this;
}
auto FuncDefNode::get_parameters()
    -> std::vector<std::shared_ptr<VarFuncNode>> & {
  return parameters;
}
auto FuncDefNode::set_parameters(
    const std::vector<std::shared_ptr<VarFuncNode>> &param) -> FuncDefNode & {
  this->parameters = param;
  return *this;
}
auto FuncDefNode::add_parameter(std::shared_ptr<VarFuncNode> const &parameter)
    -> FuncDefNode & {
  this->parameters.push_back(parameter);
  return *this;
}
auto FuncDefNode::get_block_id() const -> int { return block_id; }
auto FuncDefNode::set_block_id(int id) -> FuncDefNode & {
  this->block_id = id;
  return *this;
}
auto FuncDefNode::get_local_var_total_size() const -> int {
  return locak_var_total_size;
}
auto FuncDefNode::set_local_var_total_size(int size) -> FuncDefNode & {
  this->locak_var_total_size = size;
  return *this;
}
auto FuncDefNode::get_return_label_id() const -> int { return return_label_id; }
auto FuncDefNode::set_return_label_id(int return_label) -> FuncDefNode & {
  this->return_label_id = return_label;
  return *this;
}
// RefNode
RefNode::RefNode() : Node(), name() {}
RefNode::RefNode(const std::shared_ptr<Node> &node, type::NODE_TYPE type,
                 std::string name)
    : Node(node, type), name(name) {}
auto RefNode::get_name() const -> std::string_view { return name; }
auto RefNode::set_name(const std::string &name) -> RefNode & {
  this->name = name;
  return *this;
}
// VarNode
VarNode::VarNode() : RefNode(), target_var(nullptr) {}
VarNode::VarNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE node_type,
                 std::string name)
    : RefNode(parent, node_type, name), target_var(nullptr) {}
auto VarNode::get_target_var() const -> const std::shared_ptr<VarTypeNode> & {
  return target_var;
}
auto VarNode::set_target_var(const std::shared_ptr<VarTypeNode> &target_var)
    -> VarNode & {
  this->target_var = target_var;
  return *this;
}
// FuncNode
FuncNode::FuncNode()
    : RefNode(), parameters(), is_runtime_func_(false), target_fun(nullptr) {}
auto FuncNode::is_runtime_func() const -> bool { return is_runtime_func_; }
auto FuncNode::set_is_runtime_func(bool is_runtime_func) -> FuncNode & {
  this->is_runtime_func_ = is_runtime_func;
  return *this;
}
auto FuncNode::get_parameters() -> std::vector<std::shared_ptr<Node>> & {
  return parameters;
}
auto FuncNode::set_parameters(
    const std::vector<std::shared_ptr<Node>> &parameter) -> FuncNode & {
  this->parameters = parameter;
  return *this;
}
auto FuncNode::add_parameter(std::shared_ptr<Node> const &parameter)
    -> FuncNode & {
  this->parameters.push_back(parameter);
  return *this;
}
auto FuncNode::get_target_func() const -> const std::shared_ptr<FuncDefNode> & {
  return target_fun;
}
auto FuncNode::set_target_func(const std::shared_ptr<FuncDefNode> &target_fun)
    -> FuncNode & {
  this->target_fun = target_fun;
  return *this;
}
// StmtNode
StmtNode::StmtNode() : Node(), stmt(nullptr) {}
auto StmtNode::get_stmt() -> std::shared_ptr<Node> & { return stmt; }
auto StmtNode::set_stmt(const std::shared_ptr<Node> &stmt) -> StmtNode & {
  this->stmt = stmt;
  return *this;
}
// ParenthesisNode
ParenthesisNode::ParenthesisNode() : Node(), node(nullptr) {}
auto ParenthesisNode::get_node() -> std::shared_ptr<Node> & { return node; }
auto ParenthesisNode::set_node(const std::shared_ptr<Node> &node)
    -> ParenthesisNode & {
  this->node = node;
  return *this;
}
// LiteralNode
LiteralNode::LiteralNode() : Node(), value(-1) {}
LiteralNode::LiteralNode(std::shared_ptr<LiteralNode> const &literal_node)
    : Node(literal_node->get_parent(), literal_node->get_type()),
      value(literal_node->get_value()) {}
LiteralNode::LiteralNode(std::shared_ptr<Node> const &parent,
                         type::NODE_TYPE type, int value)

    : Node(parent, type), value(value) {}
auto LiteralNode::get_value() const -> int { return value; }
auto LiteralNode::set_value(int value) -> LiteralNode & {
  this->value = value;
  return *this;
}
auto LiteralNode::set_value(const char *const value) -> LiteralNode & {
  try {
    std::string val(value);
    size_t end = 0;
    int64_t _value = std::stoll(val, &end, 0);
    if (end != val.size() || _value == LONG_LONG_MAX ||
        _value == LONG_LONG_MIN) {
      std::cout << "Error at: " << __FILE__ << ", " << __LINE__;
    }
    this->value = _value;
    return *this;
  } catch (std::out_of_range &e) {
    throw std::overflow_error("Value out of range");
  }
}
FloatLiteralNode::FloatLiteralNode(
    std::shared_ptr<FloatLiteralNode> const &literal_node)
    : LiteralNode(literal_node->get_parent(), literal_node->get_type(),
                  (literal_node->get_value())) {}
auto FloatLiteralNode::set_value(const char *const value)
    -> FloatLiteralNode & {
  try {
    std::string val(value);
    float _value = std::stof(val);
    this->value = static_cast<int64_t>(_value);
  } catch (const std::invalid_argument &e) {
    std::cerr << "Invalid argument:" << e.what() << std::endl;
  } catch (const std::out_of_range &e) {
    std::cout << "Out of range: " << e.what() << std::endl;
  }
  return *this;
}
// ArrayNode
ArrayNode::ArrayNode() : Node(), array() {}
auto ArrayNode::get_array() -> std::vector<std::shared_ptr<Node>> & {
  return array;
}
auto ArrayNode::set_array(const std::vector<std::shared_ptr<Node>> &array)
    -> ArrayNode & {
  this->array = array;
  return *this;
}
auto ArrayNode::add_array(std::shared_ptr<Node> const &row) -> ArrayNode & {
  this->array.push_back(row);
  return *this;
}
// ArrayAccessNode
ArrayAccessNode::ArrayAccessNode() : Node(), indexes(), target(nullptr) {}
auto ArrayAccessNode::get_indexes() -> std::vector<std::shared_ptr<Node>> & {
  return indexes;
}
auto ArrayAccessNode::set_indexes(
    const std::vector<std::shared_ptr<Node>> &indexes) -> ArrayAccessNode & {
  this->indexes = indexes;
  return *this;
}
auto ArrayAccessNode::add_index(std::shared_ptr<Node> const &index)
    -> ArrayAccessNode & {
  this->indexes.push_back(index);
  return *this;
}
auto ArrayAccessNode::get_target() const -> const std::shared_ptr<Node> & {
  return target;
}
auto ArrayAccessNode::set_target(const std::shared_ptr<Node> &target)
    -> ArrayAccessNode & {
  this->target = target;
  return *this;
}
// OperationNode
OperationNode::OperationNode()
    : Node(), operator_type(type::OPERATION_TYPE::OP_INVALID), parameters() {}
auto OperationNode::get_op_type() const -> type::OPERATION_TYPE {
  return operator_type;
}
auto OperationNode::set_op_type(type::OPERATION_TYPE type) -> OperationNode & {
  this->operator_type = type;
  return *this;
}
auto OperationNode::get_parameters() -> std::vector<std::shared_ptr<Node>> & {
  return parameters;
}
auto OperationNode::set_parameters(
    const std::vector<std::shared_ptr<Node>> &parameters) -> OperationNode & {
  this->parameters = parameters;
  return *this;
}
auto OperationNode::add_parameter(const std::shared_ptr<Node> &node)
    -> OperationNode & {
  this->parameters.push_back(node);
  return *this;
}
auto OperationNode::reverse_parameters() -> OperationNode & {
  std::reverse(parameters.begin(), parameters.end());
  return *this;
}
// IfElseNode
IfElseNode::IfElseNode()
    : Node(), condition(nullptr), if_body(nullptr), else_body(nullptr) {}
IfElseNode::IfElseNode(std::shared_ptr<Node> const &parent,
                       type::NODE_TYPE node_type,
                       std::shared_ptr<Node> const &condition,
                       std::shared_ptr<BlockNode> const &if_body,
                       std::shared_ptr<BlockNode> const &else_body)
    : Node(parent, node_type), condition(condition), if_body(if_body),
      else_body(else_body) {}
auto IfElseNode::get_condition() const -> const std::shared_ptr<Node> & {
  return condition;
}
auto IfElseNode::set_condition(std::shared_ptr<Node> const &cond)
    -> IfElseNode & {
  this->condition = cond;
  return *this;
}
auto IfElseNode::get_if_body() const -> const std::shared_ptr<BlockNode> & {
  return if_body;
}
auto IfElseNode::get_else_body() const -> const std::shared_ptr<BlockNode> & {
  return else_body;
}
auto IfElseNode::set_if_body(const std::shared_ptr<BlockNode> &if_body)
    -> IfElseNode & {
  this->if_body = if_body;
  return *this;
}

auto IfElseNode::set_else_body(const std::shared_ptr<BlockNode> &else_body)
    -> IfElseNode & {
  this->else_body = else_body;
  return *this;
}
// WhileNode
WhileNode::WhileNode()
    : Node(), condition(nullptr), body(nullptr), continue_label_id(-1),
      break_label_id(-1) {}

WhileNode::WhileNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE type,
                     std::shared_ptr<Node> const &condition,
                     std::shared_ptr<BlockNode> const &body)
    : Node(parent, type), condition(condition), body(body),
      continue_label_id(-1), break_label_id(-1) {}
auto WhileNode::get_condition() const -> const std::shared_ptr<Node> & {
  return condition;
}
auto WhileNode::set_condition(std::shared_ptr<Node> const &cond)
    -> WhileNode & {
  this->condition = cond;
  return *this;
}
auto WhileNode::get_body() const -> const std::shared_ptr<BlockNode> & {
  return body;
}
auto WhileNode::set_body(std::shared_ptr<BlockNode> const &body)
    -> WhileNode & {
  this->body = body;
  return *this;
}
auto WhileNode::get_continue_label_id() const -> int {
  return continue_label_id;
}
auto WhileNode::set_continue_label_id(int id) -> WhileNode & {
  this->continue_label_id = id;
  return *this;
}
auto WhileNode::get_break_label_id() const -> int { return break_label_id; }
auto WhileNode::set_break_label_id(int id) -> WhileNode & {
  this->break_label_id = id;
  return *this;
}
// ContinueNode
ContinueNode::ContinueNode()
    : Node(), target(

              ) {}
auto ContinueNode::get_target() const -> const std::shared_ptr<WhileNode> & {
  return target;
}
auto ContinueNode::set_target(const std::shared_ptr<WhileNode> &target)
    -> ContinueNode & {
  this->target = target;
  return *this;
}
// BreakNode
BreakNode::BreakNode() : Node(), target(nullptr) {}
auto BreakNode::set_target(const std::shared_ptr<WhileNode> &target)
    -> BreakNode & {
  this->target = target;
  return *this;
}
auto BreakNode::get_target() const -> const std::shared_ptr<WhileNode> & {
  return target;
}
// ReturnNode
ReturnNode::ReturnNode() : Node(), return_value(nullptr), target(nullptr) {}
auto ReturnNode::get_return_value() const -> const std::shared_ptr<Node> & {
  return return_value;
}
auto ReturnNode::set_return_value(const std::shared_ptr<Node> &return_val)
    -> ReturnNode & {
  this->return_value = return_val;
  return *this;
}
auto ReturnNode::get_target() const -> const std::shared_ptr<FuncDefNode> & {
  return target;
}
auto ReturnNode::set_target(const std::shared_ptr<FuncDefNode> &target)
    -> ReturnNode & {
  this->target = target;
  return *this;
}
// BlockNode
BlockNode::BlockNode() : Node(), children(), id(-1) {}
auto BlockNode::get_children() -> std::vector<std::shared_ptr<Node>> & {
  return children;
}
auto BlockNode::get_children() const
    -> const std::vector<std::shared_ptr<Node>> & {
  return children;
}
auto BlockNode::set_children(
    const std::vector<std::shared_ptr<Node>> &childrent) -> BlockNode & {
  this->children = childrent;
  return *this;
}
auto BlockNode::add_children(std::shared_ptr<Node> const &child)
    -> BlockNode & {
  this->children.push_back(child);
  return *this;
}
auto BlockNode::get_id() const -> int { return id; }
auto BlockNode::set_id(int id) -> BlockNode & {
  this->id = id;
  return *this;
}

} // namespace compiler::node
