#pragma once
#include "type.h"
#include <cmath>
#include <concepts>
#include <cstdint>
#include <memory>
#include <pthread.h>
#include <string>
#include <string_view>
#include <type_traits>
#include <utility>
#include <vector>
namespace compiler::node {
namespace type = compiler::type;
// 前向声明
class Node; // AST每一个节点的表示
class TypeNode;
class VarTypeNode;
class BlockNode;
class DefNode;
class FuncDefNode;
class RefNode;
class VarNode;
class StmtNode;
class FuncNode;
class ArrayNode;
class ArrayAccessNode;
class WhileNode;
class LiteralNode;
class FloatLiteralNode;
class ContinueNode;
class BlockNode;
class BreakNode;
class VarFuncNode;
class IfElseNode;
class OperationNode;
// class Root;
// concept定义
template <typename T>
concept ValidType = std::is_integral_v<T> || std::is_floating_point_v<T> ||
                    std::same_as<T, std::shared_ptr<Node>> ||
                    std::constructible_from<std::shared_ptr<Node>, T>;

// 实现
class Node {
private:
  std::shared_ptr<Node> parent;
  type::NODE_TYPE type;

public:
  Node();
  Node(const Node &) = default;
  Node(Node &&) = delete;
  auto operator=(const Node &) -> Node & = default;
  auto operator=(Node &&) -> Node & = delete;
  Node(std::shared_ptr<Node> const &parent, type::NODE_TYPE type);
  virtual ~Node() = default;
  [[nodiscard]] auto get_parent() const -> const std::shared_ptr<Node> &;
  auto set_parent(const std::shared_ptr<Node> &parent) -> Node &;
  [[nodiscard]] auto get_type() const -> type::NODE_TYPE;
  auto set_type(type::NODE_TYPE type) -> Node &;
};
class TypeNode : public Node {
private:
  type::VALUE_TYPE variable_type;
  std::vector<std::shared_ptr<Node>> multi_dimension_array;

public:
  TypeNode();
  TypeNode(type::VALUE_TYPE variable_type,
           std::vector<std::shared_ptr<Node>> multi_dimension_array)
      : variable_type(variable_type),
        multi_dimension_array(std::move(multi_dimension_array)) {}
  TypeNode(const TypeNode &) = default;
  TypeNode(TypeNode &&) = delete;
  auto operator=(const TypeNode &) -> TypeNode & = default;
  auto operator=(TypeNode &&) -> TypeNode & = delete;
  template <ValidType... T>
  TypeNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE node_type,
           type::VALUE_TYPE var_type, T &&...args)
      : Node(parent, node_type), variable_type(var_type),
        multi_dimension_array() {
    (multi_dimension_array.push_back(std::forward<T>(args)), ...);
  }
  ~TypeNode() override = default;
  auto get_array() -> std::vector<std::shared_ptr<Node>> &;
  auto set_array(const std::vector<std::shared_ptr<Node>> &array) -> TypeNode &;
  auto add_array(std::shared_ptr<Node> const &array) -> TypeNode &;
  [[nodiscard]] auto get_variable_type() const -> type::VALUE_TYPE;
  auto set_variable_type(type::VALUE_TYPE type) -> TypeNode &;
};
class VarTypeNode : public Node {
private:
  std::string name;
  std::vector<std::shared_ptr<Node>> arrays;

public:
  VarTypeNode();
  VarTypeNode(const VarTypeNode &) = default;
  VarTypeNode(VarTypeNode &&) = delete;
  auto operator=(const VarTypeNode &) -> VarTypeNode & = default;
  auto operator=(VarTypeNode &&) -> VarTypeNode & = delete;
  VarTypeNode(std::string name, std::vector<std::shared_ptr<Node>> arrays)
      : name(std::move(name)), arrays(std::move(arrays)) {}
  template <ValidType... T>
  VarTypeNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE node_type,
              std::string name, T &&...args)
      : Node(parent, node_type), name(std::move(name)), arrays() {
    (arrays.push_back(std::forward<T>(args)), ...);
  }
  ~VarTypeNode() override = default;
  auto get_array() -> std::vector<std::shared_ptr<Node>> &;
  auto
  set_array(const std::vector<std::shared_ptr<Node>> &array) -> VarTypeNode &;
  auto add_array(std::shared_ptr<Node> const &array) -> VarTypeNode &;
  [[nodiscard]] auto get_name() const -> const std::string & { return name; }
  auto set_name(const std::string &name) -> VarTypeNode &;
};
class DefNode : public Node {
private:
  std::string name;

public:
  DefNode();
  DefNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE node_type,
          std::string name);
  [[nodiscard]] auto get_name() const -> const std::string &;
  auto set_name(const std::string &name) -> DefNode &;
};
class VarFuncNode : public DefNode {
private:
  std::shared_ptr<TypeNode> variable_type;            // 变量类型
  std::vector<std::shared_ptr<VarTypeNode>> var_type; // 变量定义
  std::vector<std::shared_ptr<Node>> var_values;      // 变量值
  bool is_const_;                                     // 是否常值
  bool is_global_;                                    // 全局
  int block_id;
  std::vector<int> offset; // 变量偏移
                           //
public:
  VarFuncNode();
  VarFuncNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE node_type,
              std::string name, std::shared_ptr<TypeNode> const &variable_type,
              bool is_const);

  ~VarFuncNode() override = default;
  // getters, setters
  [[nodiscard]] auto is_const() const -> bool;
  [[nodiscard]] auto is_global() const -> bool;
  [[nodiscard]] auto set_const(bool is_const) -> VarFuncNode &;
  [[nodiscard]] auto set_global(bool is_global) -> VarFuncNode &;
  auto get_variable_type() -> std::shared_ptr<TypeNode> &;
  auto set_variable_type(const std::shared_ptr<TypeNode> &variable_type)
      -> VarFuncNode &;
  auto get_var_value() -> std::vector<std::shared_ptr<Node>> &;
  auto set_var_value(const std::vector<std::shared_ptr<Node>> &var_init_value)
      -> VarFuncNode &;
  auto get_var_type() -> std::vector<std::shared_ptr<VarTypeNode>> &;
  auto set_var_type(const std::vector<std::shared_ptr<VarTypeNode>> &var_type)
      -> VarFuncNode &;
  [[nodiscard]] auto get_block_id() const -> int;
  [[nodiscard]] auto set_block_id(int block_id) -> VarFuncNode &;
  [[nodiscard]] auto get_off_set() const -> const std::vector<int> &;
  auto set_off_set(const std::vector<int> &var_off_set) -> VarFuncNode &;
  auto add_var_offset(const int off_set) -> VarFuncNode &;

  // 添加
  auto add_var_value(std::shared_ptr<Node> const &value) -> VarFuncNode &;
  auto add_var(std::shared_ptr<VarTypeNode> var_type,
               std::shared_ptr<Node> var_value) -> VarFuncNode &;

public:
  VarFuncNode(const VarFuncNode &) = default;
  VarFuncNode(VarFuncNode &&) = delete;
  auto operator=(const VarFuncNode &) -> VarFuncNode & = default;
  auto operator=(VarFuncNode &&) -> VarFuncNode & = delete;
};
class FuncDefNode : public DefNode {
private:
  std::shared_ptr<TypeNode> return_type;
  std::shared_ptr<BlockNode> body;
  std::vector<std::shared_ptr<VarFuncNode>> parameters;
  int block_id;
  int return_label_id;
  int locak_var_total_size;

public:
  FuncDefNode();
  FuncDefNode(const FuncDefNode &) = default;
  FuncDefNode(FuncDefNode &&) = delete;
  auto operator=(const FuncDefNode &) -> FuncDefNode & = default;
  auto operator=(FuncDefNode &&) -> FuncDefNode & = delete;
  FuncDefNode(std::shared_ptr<TypeNode> return_type,
              std::shared_ptr<BlockNode> body,
              std::vector<std::shared_ptr<VarFuncNode>> parameters,
              int block_id, int return_label_id, int locak_car_total_size)
      : return_type(std::move(return_type)), body(std::move(body)),
        parameters(std::move(parameters)), block_id(block_id),
        return_label_id(return_label_id),
        locak_var_total_size(locak_car_total_size) {}
  template <ValidType... T>
  FuncDefNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE node_type,
              std::string name, std::shared_ptr<TypeNode> const &return_type,
              std::shared_ptr<BlockNode> const &body, T &&...args)
      : DefNode(parent, node_type, name), return_type(return_type), body(body),
        parameters(), block_id(-1), return_label_id(-1),
        locak_var_total_size(-1) {
    (parameters.push_back(std::forward<T>(args)), ...);
  }
  ~FuncDefNode() override = default;
  auto get_return_type() -> const std::shared_ptr<TypeNode> &;
  auto set_return_type(const std::shared_ptr<TypeNode> &type) -> FuncDefNode &;
  [[nodiscard]] auto get_body() const -> const std::shared_ptr<BlockNode> &;
  auto set_body(const std::shared_ptr<BlockNode> &body) -> FuncDefNode &;
  auto get_parameters() -> std::vector<std::shared_ptr<VarFuncNode>> &;
  auto
  set_parameters(const std::vector<std::shared_ptr<VarFuncNode>> &parameters)
      -> FuncDefNode &;
  auto
  add_parameter(std::shared_ptr<VarFuncNode> const &parameter) -> FuncDefNode &;
  [[nodiscard]] auto get_block_id() const -> int;
  auto set_block_id(int block_id) -> FuncDefNode &;
  [[nodiscard]] auto get_local_var_total_size() const -> int;
  auto set_local_var_total_size(int locak_var_total_size) -> FuncDefNode &;
  [[nodiscard]] auto get_return_label_id() const -> int;
  auto set_return_label_id(int return_label_id) -> FuncDefNode &;
};
class RefNode : public Node {
private:
  std::string name;

public:
  RefNode();

  RefNode(const RefNode &) = default;
  RefNode(RefNode &&) = delete;
  auto operator=(const RefNode &) -> RefNode & = default;
  auto operator=(RefNode &&) -> RefNode & = delete;
  RefNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE node_type,
          std::string name);
  [[nodiscard]] auto get_name() const -> std::string_view;
  auto set_name(const std::string &name) -> RefNode &;
  ~RefNode() override = default;
};
class VarNode : public RefNode {
  std::shared_ptr<VarTypeNode> target_var;

public:
  VarNode();
  VarNode(const VarNode &) = default;
  VarNode(VarNode &&) = delete;
  auto operator=(const VarNode &) -> VarNode & = default;
  auto operator=(VarNode &&) -> VarNode & = delete;
  VarNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE node_type,
          std::string name);
  ~VarNode() override = default;
  [[nodiscard]] auto
  get_target_var() const -> const std::shared_ptr<VarTypeNode> &;
  auto
  set_target_var(const std::shared_ptr<VarTypeNode> &target_var) -> VarNode &;
};
class FuncNode : public RefNode {
  std::vector<std::shared_ptr<Node>> parameters;
  bool is_runtime_func_;
  std::shared_ptr<FuncDefNode> target_fun;

public:
  FuncNode();
  FuncNode(const FuncNode &) = default;
  FuncNode(FuncNode &&) = delete;
  auto operator=(const FuncNode &) -> FuncNode & = default;
  auto operator=(FuncNode &&) -> FuncNode & = delete;
  FuncNode(std::vector<std::shared_ptr<Node>> parameters, bool is_runtime_func,
           std::shared_ptr<FuncDefNode> target_fun)
      : parameters(std::move(parameters)), is_runtime_func_(is_runtime_func),
        target_fun(std::move(target_fun)) {}
  template <ValidType... T>
  FuncNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE node_type,
           std::string name, T &&...args)
      : RefNode(parent, node_type, name), parameters(), is_runtime_func_(false),
        target_fun(nullptr) {
    ((parameters.push_back(std::forward<T>(args))), ...);
  }
  ~FuncNode() override = default;
  auto get_parameters() -> std::vector<std::shared_ptr<Node>> &;
  auto set_parameters(const std::vector<std::shared_ptr<Node>> &parameters)
      -> FuncNode &;
  auto add_parameter(std::shared_ptr<Node> const &parameter) -> FuncNode &;
  [[nodiscard]] auto is_runtime_func() const -> bool;
  auto set_is_runtime_func(bool is_runtime) -> FuncNode &;
  [[nodiscard]] auto
  get_target_func() const -> const std::shared_ptr<FuncDefNode> &;
  auto set_target_func(const std::shared_ptr<FuncDefNode> &target_func)
      -> FuncNode &;
};
class StmtNode : public Node {
  std::shared_ptr<Node> stmt;

public:
  StmtNode();
  StmtNode(const StmtNode &) = default;
  StmtNode(StmtNode &&) = delete;
  auto operator=(const StmtNode &) -> StmtNode & = default;
  auto operator=(StmtNode &&) -> StmtNode & = delete;
  StmtNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE node_type,
           std::shared_ptr<Node> const &stmt)
      : Node(parent, node_type), stmt(stmt) {};
  ~StmtNode() override = default;
  auto get_stmt() -> std::shared_ptr<Node> &;
  auto set_stmt(const std::shared_ptr<Node> &stmt) -> StmtNode &;
};
class ParenthesisNode : public Node {
  std::shared_ptr<Node> node;

public:
  ParenthesisNode();
  ParenthesisNode(const ParenthesisNode &) = default;
  ParenthesisNode(ParenthesisNode &&) = delete;
  auto operator=(const ParenthesisNode &) -> ParenthesisNode & = default;
  auto operator=(ParenthesisNode &&) -> ParenthesisNode & = delete;
  ParenthesisNode(std::shared_ptr<Node> const &parent,
                  type::NODE_TYPE node_type, std::shared_ptr<Node> const &node)
      : Node(node, node_type), node(node) {};
  ~ParenthesisNode() override = default;
  auto get_node() -> std::shared_ptr<Node> &;
  auto set_node(const std::shared_ptr<Node> &node) -> ParenthesisNode &;
};
class LiteralNode : public Node {
protected:
  int value;

public:
  LiteralNode();
  LiteralNode(const LiteralNode &) = default;
  LiteralNode(LiteralNode &&) = delete;
  auto operator=(const LiteralNode &) -> LiteralNode & = default;
  auto operator=(LiteralNode &&) -> LiteralNode & = delete;
  explicit LiteralNode(std::shared_ptr<LiteralNode> const &literal_node);
  LiteralNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE node_type,
              int value);
  ~LiteralNode() override = default;
  [[nodiscard]] auto get_value() const -> int;
  auto set_value(int value) -> LiteralNode &;
  auto set_value(const char *const value) -> LiteralNode &;
};
class FloatLiteralNode : public LiteralNode {
public:
  FloatLiteralNode() = default;
  explicit FloatLiteralNode(
      std::shared_ptr<FloatLiteralNode> const &literal_node);
  FloatLiteralNode(std::shared_ptr<Node> const &parent,
                   type::NODE_TYPE node_type, float value)
      : LiteralNode(parent, node_type, static_cast<int64_t>(value)) {};
  ~FloatLiteralNode() override = default;
  [[nodiscard]] auto get_value() const -> float {
    return static_cast<float>(value);
  }
  auto set_value(float value) -> FloatLiteralNode & {
    this->value = static_cast<int64_t>(value);
    return *this;
  }
  auto set_value(const char *const value) -> FloatLiteralNode &;
};
class ArrayNode : public Node {
  std::vector<std::shared_ptr<Node>> array;

public:
  ArrayNode();
  ArrayNode(const ArrayNode &) = default;
  ArrayNode(ArrayNode &&) = delete;
  auto operator=(const ArrayNode &) -> ArrayNode & = default;
  auto operator=(ArrayNode &&) -> ArrayNode & = delete;
  template <ValidType... T>
  ArrayNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE node_type,
            T &&...args) {
    (array.push_back(std::forward<T>(args)), ...);
  }
  ~ArrayNode() override = default;
  auto get_array() -> std::vector<std::shared_ptr<Node>> &;
  auto
  set_array(const std::vector<std::shared_ptr<Node>> &array) -> ArrayNode &;
  auto add_array(std::shared_ptr<Node> const &row) -> ArrayNode &;
};
class ArrayAccessNode : public Node {
  std::vector<std::shared_ptr<Node>> indexes;
  std::shared_ptr<Node> target;

public:
  ArrayAccessNode();
  ArrayAccessNode(const ArrayAccessNode &) = default;
  ArrayAccessNode(ArrayAccessNode &&) = delete;
  auto operator=(const ArrayAccessNode &) -> ArrayAccessNode & = default;
  auto operator=(ArrayAccessNode &&) -> ArrayAccessNode & = delete;
  ArrayAccessNode(std::vector<std::shared_ptr<Node>> indexes,
                  std::shared_ptr<Node> target)
      : indexes(std::move(indexes)), target(std::move(target)) {}
  template <ValidType... T>
  ArrayAccessNode(std::shared_ptr<Node> const &parent,
                  type::NODE_TYPE node_type,
                  std::shared_ptr<Node> const &target, T &&...args)
      : Node(parent, node_type), target(target), indexes() {
    (indexes.push_back(std::forward<T>(args)), ...);
  }
  ~ArrayAccessNode() override = default;
  auto get_indexes() -> std::vector<std::shared_ptr<Node>> &;
  auto set_indexes(const std::vector<std::shared_ptr<Node>> &indexes)
      -> ArrayAccessNode &;
  auto add_index(std::shared_ptr<Node> const &index) -> ArrayAccessNode &;
  [[nodiscard]] auto get_target() const -> const std::shared_ptr<Node> &;
  auto set_target(const std::shared_ptr<Node> &target) -> ArrayAccessNode &;
};
class OperationNode : public Node {
  type::OPERATION_TYPE operator_type;
  std::vector<std::shared_ptr<Node>> parameters;

public:
  OperationNode();
  template <ValidType... T>
  OperationNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE node_type,
                type::OPERATION_TYPE operator_type, T &&...args)
      : Node(parent, node_type), operator_type(operator_type), parameters() {
    (parameters.push_back(std::forward<T>(args)), ...);
  }
  [[nodiscard]] auto get_op_type() const -> type::OPERATION_TYPE;
  auto set_op_type(type::OPERATION_TYPE type) -> OperationNode &;
  auto get_parameters() -> std::vector<std::shared_ptr<Node>> &;
  auto set_parameters(const std::vector<std::shared_ptr<Node>> &parameters)
      -> OperationNode &;
  auto add_parameter(std::shared_ptr<Node> const &parameter) -> OperationNode &;
  auto reverse_parameters() -> OperationNode &;
};
class IfElseNode : public Node {
  std::shared_ptr<Node> condition;
  std::shared_ptr<BlockNode> if_body;
  std::shared_ptr<BlockNode> else_body;

public:
  IfElseNode();
  IfElseNode(const IfElseNode &) = default;
  IfElseNode(IfElseNode &&) = delete;
  auto operator=(const IfElseNode &) -> IfElseNode & = default;
  auto operator=(IfElseNode &&) -> IfElseNode & = delete;
  IfElseNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE node_type,
             std::shared_ptr<Node> const &condition,
             std::shared_ptr<BlockNode> const &if_body,
             std::shared_ptr<BlockNode> const &else_body);
  ~IfElseNode() override = default;
  [[nodiscard]] auto get_condition() const -> const std::shared_ptr<Node> &;
  auto set_condition(std::shared_ptr<Node> const &condition) -> IfElseNode &;
  [[nodiscard]] auto get_if_body() const -> const std::shared_ptr<BlockNode> &;
  [[nodiscard]] auto
  get_else_body() const -> const std::shared_ptr<BlockNode> &;
  auto set_if_body(const std::shared_ptr<BlockNode> &if_body) -> IfElseNode &;
  auto
  set_else_body(const std::shared_ptr<BlockNode> &else_body) -> IfElseNode &;
};
class WhileNode : public Node {
  std::shared_ptr<Node> condition;
  std::shared_ptr<BlockNode> body;
  int continue_label_id;
  int break_label_id;

public:
  WhileNode();
  WhileNode(const WhileNode &) = default;
  WhileNode(WhileNode &&) = delete;
  auto operator=(const WhileNode &) -> WhileNode & = default;
  auto operator=(WhileNode &&) -> WhileNode & = delete;
  WhileNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE type,
            std::shared_ptr<Node> const &condition,
            std::shared_ptr<BlockNode> const &body);
  ~WhileNode() override = default;
  [[nodiscard]] auto get_condition() const -> const std::shared_ptr<Node> &;
  [[nodiscard]] auto get_body() const -> const std::shared_ptr<BlockNode> &;
  auto set_condition(std::shared_ptr<Node> const &condition) -> WhileNode &;
  auto set_body(std::shared_ptr<BlockNode> const &body) -> WhileNode &;
  [[nodiscard]] auto get_continue_label_id() const -> int;
  auto set_continue_label_id(int id) -> WhileNode &;
  [[nodiscard]] auto get_break_label_id() const -> int;
  auto set_break_label_id(int id) -> WhileNode &;
};
class ContinueNode : public Node {
  std::shared_ptr<WhileNode> target;

public:
  ContinueNode();
  ContinueNode(const ContinueNode &) = default;
  ContinueNode(ContinueNode &&) = delete;
  auto operator=(const ContinueNode &) -> ContinueNode & = default;
  auto operator=(ContinueNode &&) -> ContinueNode & = delete;
  ContinueNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE type)
      : Node(parent, type) {};
  ~ContinueNode() override = default;
  [[nodiscard]] auto get_target() const -> const std::shared_ptr<WhileNode> &;
  auto set_target(const std::shared_ptr<WhileNode> &target) -> ContinueNode &;
};
class BreakNode : public Node {
  std::shared_ptr<WhileNode> target;

public:
  BreakNode();
  BreakNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE type)
      : Node(parent, type), target() {};
  [[nodiscard]] auto get_target() const -> const std::shared_ptr<WhileNode> &;
  auto set_target(const std::shared_ptr<WhileNode> &target) -> BreakNode &;
};
class ReturnNode : public Node {
  std::shared_ptr<Node> return_value;
  std::shared_ptr<FuncDefNode> target;

public:
  ReturnNode();
  ReturnNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE type,
             std::shared_ptr<Node> const &return_value)
      : Node(parent, type), return_value(return_value) {};
  [[nodiscard]] auto get_return_value() const -> const std::shared_ptr<Node> &;
  [[nodiscard]] auto get_target() const -> const std::shared_ptr<FuncDefNode> &;
  auto
  set_return_value(const std::shared_ptr<Node> &return_value) -> ReturnNode &;
  auto set_target(const std::shared_ptr<FuncDefNode> &target) -> ReturnNode &;
};
class BlockNode : public Node {
  std::vector<std::shared_ptr<Node>> children;
  int id;

public:
  BlockNode();
  BlockNode(const BlockNode &) = default;
  BlockNode(BlockNode &&) = delete;
  auto operator=(const BlockNode &) -> BlockNode & = default;
  auto operator=(BlockNode &&) -> BlockNode & = delete;
  template <ValidType... T>
  BlockNode(std::shared_ptr<Node> const &parent, type::NODE_TYPE type,
            T &&...args)
      : Node(parent, type), children(), id(-1) {
    (children.push_back(std::forward<T>(args)), ...);
  }
  ~BlockNode() override = default;
  [[nodiscard]] auto get_children() -> std::vector<std::shared_ptr<Node>> &;
  [[nodiscard]] auto
  get_children() const -> const std::vector<std::shared_ptr<Node>> &;
  auto set_children(const std::vector<std::shared_ptr<Node>> &children)
      -> BlockNode &;
  auto add_children(std::shared_ptr<Node> const &child) -> BlockNode &;
  [[nodiscard]] auto get_id() const -> int;
  auto set_id(int id) -> BlockNode &;
};
class Root : public Node {
  std::vector<std::shared_ptr<Node>> body;

public:
  Root();
  Root(const Root &) = default;
  Root(Root &&) = delete;
  auto operator=(const Root &) -> Root & = default;
  auto operator=(Root &&) -> Root & = delete;
  ~Root() override = default;
  auto get_body() -> std::vector<std::shared_ptr<Node>> & { return body; }
  auto set_body(const std::vector<std::shared_ptr<Node>> &body) -> Root & {
    this->body = body;
    return *this;
  }
};
} // namespace compiler::node
