/*
 *存放类型定义
 */
#pragma once
#include "fmt/core.h"
#include <array>
#include <compare>
#include <cstdint>
#include <cstring>
#include <fmt/format.h>
#include <stdexcept>
#include <string>
#include <string_view>
#include <unordered_map>
// 以F开头，表示返回值
#define STRINGIFY(x) #x
#define TO_STRING(x) STRINGIFY(x)
namespace compiler::type {
enum class VALUE_TYPE : uint8_t { V_INVALID, V_VOID, V_INT, V_FLOAT };
static VALUE_TYPE str_to_value_type(const std::string &str) {
  if (str == "V_INVALID")
    return VALUE_TYPE::V_INVALID;
  if (str == "V_INT")
    return VALUE_TYPE::V_INT;
  if (str == "V_VOID")
    return VALUE_TYPE::V_VOID;
  if (str == "V_FLOAT")
    return VALUE_TYPE::V_FLOAT;
}
static std::string value_type_to_string(VALUE_TYPE type) {
  switch (type) {
  case VALUE_TYPE::V_INT:
    return TO_STRING(V_INT);
  case VALUE_TYPE::V_FLOAT:
    return TO_STRING(V_FLOAT);
  case VALUE_TYPE::V_VOID:
    return TO_STRING(V_VOID);
  case VALUE_TYPE::V_INVALID:
    return TO_STRING(V_INVALID);
  }
}
constexpr static bool operator!(VALUE_TYPE type) {
  return type == VALUE_TYPE::V_INVALID;
}
constexpr static std::strong_ordering operator<=>(VALUE_TYPE a, VALUE_TYPE b) {
  return static_cast<int>(a) <=> static_cast<int>(b);
}
// // 以T开头，Token类型
enum class TOKEN_TYPE : uint8_t {
  T_INVALID = 0,
  T_OPERATOR,
  T_SYMBOL,
  T_HEX_DIGIT,
  T_OCT_DIGIT,
  T_DEC_DIGIT,
  T_CONST,
  T_INT,
  T_FLOAT,
  T_VOID,
  T_IF,
  T_ELSE,
  T_WHILE,
  T_CONTINUE,
  T_BREAK,
  T_RETURN,
  T_IDENTIFIER
};

constexpr static bool operator!(TOKEN_TYPE type) {
  return type == TOKEN_TYPE::T_INVALID;
}
constexpr static auto token_type_to_val_type(TOKEN_TYPE type) -> VALUE_TYPE {
  if (type == TOKEN_TYPE::T_INT)
    return VALUE_TYPE::V_INT;
  if (type == TOKEN_TYPE::T_FLOAT)
    return VALUE_TYPE::V_FLOAT;
  if (type == TOKEN_TYPE::T_VOID)
    return VALUE_TYPE::V_VOID;
  else
    return VALUE_TYPE::V_INVALID;
}
// TOKEN_TYPE转string
static auto token_type_to_string(TOKEN_TYPE type) -> std::string {
  switch (type) {
  case TOKEN_TYPE::T_INVALID:
    return (TO_STRING(T_INVALID));
  case TOKEN_TYPE::T_OPERATOR:
    return (TO_STRING(T_OPERATOR));
  case TOKEN_TYPE::T_SYMBOL:
    return (TO_STRING(T_SYMBOL));
  case TOKEN_TYPE::T_HEX_DIGIT:
    return (TO_STRING(T_HEX_DIGIT));
  case TOKEN_TYPE::T_OCT_DIGIT:
    return (TO_STRING(T_OCT_DIGIT));
  case TOKEN_TYPE::T_DEC_DIGIT:
    return (TO_STRING(T_DEC_DIGIT));
  case TOKEN_TYPE::T_CONST:
    return (TO_STRING(T_CONST));
  case TOKEN_TYPE::T_INT:
    return (TO_STRING(T_INT));
  case TOKEN_TYPE::T_FLOAT:
    return (TO_STRING(T_FLOAT));
  case TOKEN_TYPE::T_VOID:
    return (TO_STRING(T_VOID));
  case TOKEN_TYPE::T_IF:
    return (TO_STRING(T_IF));
  case TOKEN_TYPE::T_ELSE:
    return (TO_STRING(T_ELSE));
  case TOKEN_TYPE::T_WHILE:
    return (TO_STRING(T_WHILE));
  case TOKEN_TYPE::T_CONTINUE:
    return (TO_STRING(T_CONTINUE));
  case TOKEN_TYPE::T_BREAK:
    return (TO_STRING(T_BREAK));
  case TOKEN_TYPE::T_RETURN:
    return (TO_STRING(T_RETURN));
  case TOKEN_TYPE::T_IDENTIFIER:
    return (TO_STRING(T_IDENTIFIER));
  default: {
    return (TO_STRING(T_INVALID));
  }
  }
}
// str 转TOKEN_TYPE
static auto str_to_token_type(const char *str) -> TOKEN_TYPE {
  if (!strcmp("const", str)) {
    return TOKEN_TYPE::T_CONST;
  }
  if (!strcmp("int", str)) {
    return TOKEN_TYPE::T_INT;
  }
  if (!strcmp("float", str)) {
    return TOKEN_TYPE::T_FLOAT;
  }
  if (!strcmp("void", str)) {
    return TOKEN_TYPE::T_VOID;
  }
  if (!strcmp("if", str)) {
    return TOKEN_TYPE::T_IF;
  }
  if (!strcmp("else", str)) {
    return TOKEN_TYPE::T_ELSE;
  }
  if (!strcmp("while", str)) {
    return TOKEN_TYPE::T_WHILE;
  }
  if (!strcmp("continue", str)) {
    return TOKEN_TYPE::T_CONTINUE;
  }
  if (!strcmp("break", str)) {
    return TOKEN_TYPE::T_BREAK;
  }
  if (!strcmp("return", str)) {
    return TOKEN_TYPE::T_RETURN;
  } else {
    return TOKEN_TYPE::T_IDENTIFIER;
  }
  throw std::runtime_error("Error to transfer TOKEN_TYPE.");
}
// Type of each node
enum class NODE_TYPE : uint8_t {
  N_INVALID = 0,
  N_VAR_TYPE,
  N_DEF,
  N_VAR_DEF,
  N_FUNC_DEF,
  N_TYPE,
  N_VAR,
  N_REF,
  N_FUNC,
  N_STMT,
  N_PARENTHESIS,
  N_LITERAL,
  N_FLOAT_LITERAL,
  N_ARRAY,
  N_ARRAY_INDEX,
  N_OPERATION,
  N_IF_ELSE,
  N_WHILE,
  N_CONTINUE,
  N_BREAK,
  N_RETURN,
  N_BLOCK,
  N_PIN
};
static NODE_TYPE str_to_node_type(const std::string &str) {
  static const std::unordered_map<std::string, NODE_TYPE> str_to_node_type_map =
      {{"N_INVALID", NODE_TYPE::N_INVALID},
       {"N_VAR_TYPE", NODE_TYPE::N_VAR_TYPE},
       {"N_DEF", NODE_TYPE::N_DEF},
       {"N_VAR_DEF", NODE_TYPE::N_VAR_DEF},
       {"N_FUNC_DEF", NODE_TYPE::N_FUNC_DEF},
       {"N_TYPE", NODE_TYPE::N_TYPE},
       {"N_VAR", NODE_TYPE::N_VAR},
       {"N_REF", NODE_TYPE::N_REF},
       {"N_FUNC", NODE_TYPE::N_FUNC},
       {"N_STMT", NODE_TYPE::N_STMT},
       {"N_PARENTHESIS", NODE_TYPE::N_PARENTHESIS},
       {"N_LITERAL", NODE_TYPE::N_LITERAL},
       {"N_FLOAT_LITERAL", NODE_TYPE::N_FLOAT_LITERAL},
       {"N_ARRAY", NODE_TYPE::N_ARRAY},
       {"N_ARRAY_INDEX", NODE_TYPE::N_ARRAY_INDEX},
       {"N_OPERATION", NODE_TYPE::N_OPERATION},
       {"N_IF_ELSE", NODE_TYPE::N_IF_ELSE},
       {"N_WHILE", NODE_TYPE::N_WHILE},
       {"N_CONTINUE", NODE_TYPE::N_CONTINUE},
       {"N_BREAK", NODE_TYPE::N_BREAK},
       {"N_RETURN", NODE_TYPE::N_RETURN},
       {"N_BLOCK", NODE_TYPE::N_BLOCK},
       {"N_PIN", NODE_TYPE::N_PIN}};

  auto it = str_to_node_type_map.find(str);
  if (it != str_to_node_type_map.end()) {
    return it->second;
  } else {
    throw std::invalid_argument("Invalid node type string: " + str);
  }
}
static std::string node_type_to_string(NODE_TYPE type) {
  switch (type) {
  case NODE_TYPE::N_INVALID:
    return "N_INVALID";
  case NODE_TYPE::N_VAR_TYPE:
    return "N_VAR_TYPE";
  case NODE_TYPE::N_DEF:
    return "N_DEF";
  case NODE_TYPE::N_VAR_DEF:
    return "N_VAR_DEF";
  case NODE_TYPE::N_FUNC_DEF:
    return "N_FUNC_DEF";
  case NODE_TYPE::N_TYPE:
    return "N_TYPE";
  case NODE_TYPE::N_VAR:
    return "N_VAR";
  case NODE_TYPE::N_REF:
    return "N_REF";
  case NODE_TYPE::N_FUNC:
    return "N_FUNC";
  case NODE_TYPE::N_STMT:
    return "N_STMT";
  case NODE_TYPE::N_PARENTHESIS:
    return "N_PARENTHESIS";
  case NODE_TYPE::N_LITERAL:
    return "N_LITERAL";
  case NODE_TYPE::N_FLOAT_LITERAL:
    return "N_FLOAT_LITERAL";
  case NODE_TYPE::N_ARRAY:
    return "N_ARRAY";
  case NODE_TYPE::N_ARRAY_INDEX:
    return "N_ARRAY_INDEX";
  case NODE_TYPE::N_OPERATION:
    return "N_OPERATION";
  case NODE_TYPE::N_IF_ELSE:
    return "N_IF_ELSE";
  case NODE_TYPE::N_WHILE:
    return "N_WHILE";
  case NODE_TYPE::N_CONTINUE:
    return "N_CONTINUE";
  case NODE_TYPE::N_BREAK:
    return "N_BREAK";
  case NODE_TYPE::N_RETURN:
    return "N_RETURN";
  case NODE_TYPE::N_BLOCK:
    return "N_BLOCK";
  case NODE_TYPE::N_PIN:
    return "N_PIN";
  default:
    return "N_INVALID";
  }
}
enum class OPERATION_TYPE : uint8_t {
  OP_INVALID = 0,
  OP_NONE,
  OP_UNARY_ADD,   //    自增
  OP_UNARY_MINUS, // 自减
  OP_LOGICAL_NOT, // 逻辑非
  OP_LOGICAL_AND,
  OP_LOGICAL_OR,
  OP_ADD,
  OP_SUB,
  OP_MULT,
  OP_DIV,
  OP_MOD,
  OP_GREATER,
  OP_LESS,
  OP_GE,
  OP_LE,
  OP_EQUAL,
  OP_NE,
  OP_ASSIGNMENT
};
static auto str_to_op_type(const char *const op,
                           bool is_unary) -> OPERATION_TYPE {
  if (is_unary) {
    if (!strcmp("+", op))
      return OPERATION_TYPE::OP_UNARY_ADD;
    if (!strcmp("-", op))
      return OPERATION_TYPE::OP_UNARY_MINUS;
    if (!strcmp("!", op))
      return OPERATION_TYPE::OP_LOGICAL_NOT;

  } else {
    if (!strcmp("+", op))
      return OPERATION_TYPE::OP_ADD;
    if (!strcmp("-", op))
      return OPERATION_TYPE::OP_SUB;
    if (!strcmp("*", op))
      return OPERATION_TYPE::OP_MULT;
    if (!strcmp("/", op))
      return OPERATION_TYPE::OP_DIV;
    if (!strcmp("%", op))
      return OPERATION_TYPE::OP_MOD;
    if (!strcmp(">", op))
      return OPERATION_TYPE::OP_GREATER;
    if (!strcmp(">=", op))
      return OPERATION_TYPE::OP_GE;
    if (!strcmp("<", op))
      return OPERATION_TYPE::OP_LESS;
    if (!strcmp("<=", op))
      return OPERATION_TYPE::OP_LE;
    if (!strcmp("==", op))
      return OPERATION_TYPE::OP_EQUAL;
    if (!strcmp("=", op))
      return OPERATION_TYPE::OP_ASSIGNMENT;
    if (!strcmp("!=", op))
      return OPERATION_TYPE::OP_NE;
    if (!strcmp("&&", op))
      return OPERATION_TYPE::OP_LOGICAL_AND;
    if (!strcmp("||", op))
      return OPERATION_TYPE::OP_LOGICAL_OR;
  }
  return OPERATION_TYPE::OP_INVALID;
}

static auto op_type_to_string(OPERATION_TYPE op_type) -> const char * {
  switch (op_type) {
  case OPERATION_TYPE::OP_UNARY_ADD:
    return "+";
  case OPERATION_TYPE::OP_UNARY_MINUS:
    return "-";
  case OPERATION_TYPE::OP_LOGICAL_NOT:
    return "!";
  case OPERATION_TYPE::OP_ADD:
    return "+";
  case OPERATION_TYPE::OP_SUB:
    return "-";
  case OPERATION_TYPE::OP_MULT:
    return "*";
  case OPERATION_TYPE::OP_DIV:
    return "/";
  case OPERATION_TYPE::OP_MOD:
    return "%";
  case OPERATION_TYPE::OP_GREATER:
    return ">";
  case OPERATION_TYPE::OP_GE:
    return ">=";
  case OPERATION_TYPE::OP_LESS:
    return "<";
  case OPERATION_TYPE::OP_LE:
    return "<=";
  case OPERATION_TYPE::OP_EQUAL:
    return "==";
  case OPERATION_TYPE::OP_ASSIGNMENT:
    return "=";
  case OPERATION_TYPE::OP_NE:
    return "!=";
  case OPERATION_TYPE::OP_LOGICAL_AND:
    return "&&";
  case OPERATION_TYPE::OP_LOGICAL_OR:
    return "||";
  default:
    return "INVALID_BINARY_OP";
  }
}
// 结合性
enum class ASSOCIATIVITY : uint8_t {
  NONE,
  LEFT,
  RIGHT,
};
constexpr static auto operator==(ASSOCIATIVITY a, ASSOCIATIVITY b) -> bool {
  return static_cast<int>(a) == static_cast<int>(b);
}
constexpr static auto operator<(ASSOCIATIVITY a, ASSOCIATIVITY b) -> bool {
  return static_cast<int>(a) < static_cast<int>(b);
}
constexpr static auto operator>(ASSOCIATIVITY a, ASSOCIATIVITY b) -> bool {
  return static_cast<int>(a) > static_cast<int>(b);
}
// 飞船运算符简化重载
constexpr static auto operator<=>(ASSOCIATIVITY a,
                                  ASSOCIATIVITY b) -> std::strong_ordering {
  return static_cast<int>(a) <=> static_cast<int>(b);
}
// 优先级
enum class PRECEDENCE : uint8_t {
  TRVIAL,
  LOW,
  MID_LOW,
  MID,
  MID_HIGH,
  HIGH,
  TOP_HIGH,
  TOP
};
constexpr static auto operator<=>(PRECEDENCE a,
                                  PRECEDENCE b) -> std::strong_ordering {
  return static_cast<int>(a) <=> static_cast<int>(b);
}
// 每一个操作符的信息
struct OperatorInfo {
  OPERATION_TYPE operator_type;
  // 优先级
  PRECEDENCE precedence;
  // 参数数量
  int parameter_number;
  ASSOCIATIVITY associativity;
  // constexpr OperatorInfo(OPERATION_TYPE type, PRECEDENCE prec, int paranum,
  //                        ASSOCIATIVITY assoc)
  //     : operator_type(type), precedence(prec), parameter_number(paranum),
  //       associativity(assoc) {}
};

// 关键字
static constexpr std::array<std::string_view, 10> sysy_keyword = {
    "const", "int",   "float",    "void",  "if",
    "else",  "while", "continue", "break", "return"};
// 操作符
static constexpr std::array<std::string_view, 15> sysy_operator = {
    "+", "-",  "!",  "*",  "/",  "%",  ">", ">=",
    "<", "<=", "==", "!=", "&&", "||", "="};
// 分隔符
static constexpr std::array<std::string_view, 8> sysy_delimeter = {
    ";", "(", ")", "[", "]", "{", "}", ","};
// 多字符符号
static constexpr std::array<std::string_view, 6> sysy_multi_letter_symbol = {
    ">=", "<=", "==", "!=", "&&", "||"};
// 单字符符号
static constexpr std::array<std::string_view, 17> sysy_single_letter_symbol{
    "+", "-", "*", "/", "%", "!", ">", "<", "=",
    ";", ",", "[", "]", "{", "}", "(", ")"};
// 运行时库函数
static constexpr std::array<std::string_view, 14> sysy_runtime_func = {
    "getint",   "getfloat",        "getch",          "getarray", "putint",
    "putch",    "putfloat",        "putfarray",      "putf",     "starttime",
    "stoptime", "_sysy_starttime", "_sysy_stoptime", "malloc"};
static constexpr auto sysy_keyword_len = sysy_keyword.size();
static constexpr auto sysy_operator_len = sysy_operator.size();
static constexpr auto sysy_delimeter_len = sysy_delimeter.size();
static constexpr auto sysy_multi_letter_symbol_len =
    sysy_multi_letter_symbol.size();
static constexpr auto sysy_single_letter_symbol_len =
    sysy_single_letter_symbol.size();
static constexpr auto sysy_runtime_func_len = sysy_runtime_func.size();
static constexpr std::array<std::string_view, 3> sysy_comment = {"//", "/*",
                                                                 "*/"};
static constexpr std::array<OperatorInfo, 17> sysy_operator_infos = {
    OperatorInfo{OPERATION_TYPE::OP_UNARY_ADD, PRECEDENCE::TOP, 1,
                 ASSOCIATIVITY::RIGHT},
    OperatorInfo{OPERATION_TYPE::OP_UNARY_MINUS, PRECEDENCE::TOP, 1,
                 ASSOCIATIVITY::RIGHT},
    OperatorInfo{OPERATION_TYPE::OP_LOGICAL_NOT, PRECEDENCE::TOP, 1,
                 ASSOCIATIVITY::RIGHT},
    OperatorInfo{OPERATION_TYPE::OP_MULT, PRECEDENCE::TOP_HIGH, 2,
                 ASSOCIATIVITY::LEFT},
    OperatorInfo{OPERATION_TYPE::OP_DIV, PRECEDENCE::TOP_HIGH, 2,
                 ASSOCIATIVITY::LEFT},
    OperatorInfo{OPERATION_TYPE::OP_MOD, PRECEDENCE::TOP_HIGH, 2,
                 ASSOCIATIVITY::LEFT},
    OperatorInfo{OPERATION_TYPE::OP_ADD, PRECEDENCE::HIGH, 2,
                 ASSOCIATIVITY::LEFT},
    OperatorInfo{OPERATION_TYPE::OP_SUB, PRECEDENCE::HIGH, 2,
                 ASSOCIATIVITY::LEFT},
    OperatorInfo{OPERATION_TYPE::OP_GREATER, PRECEDENCE::MID_HIGH, 2,
                 ASSOCIATIVITY::LEFT},
    OperatorInfo{OPERATION_TYPE::OP_GE, PRECEDENCE::MID_HIGH, 2,
                 ASSOCIATIVITY::LEFT},
    OperatorInfo{OPERATION_TYPE::OP_LESS, PRECEDENCE::MID_HIGH, 2,
                 ASSOCIATIVITY::LEFT},
    OperatorInfo{OPERATION_TYPE::OP_LE, PRECEDENCE::MID_HIGH, 2,
                 ASSOCIATIVITY::LEFT},
    OperatorInfo{OPERATION_TYPE::OP_EQUAL, PRECEDENCE::MID, 2,
                 ASSOCIATIVITY::LEFT},
    OperatorInfo{OPERATION_TYPE::OP_NE, PRECEDENCE::MID, 2,
                 ASSOCIATIVITY::LEFT},
    OperatorInfo{OPERATION_TYPE::OP_LOGICAL_AND, PRECEDENCE::MID_LOW, 2,
                 ASSOCIATIVITY::LEFT},
    OperatorInfo{OPERATION_TYPE::OP_LOGICAL_OR, PRECEDENCE::LOW, 2,
                 ASSOCIATIVITY::LEFT},
    OperatorInfo{OPERATION_TYPE::OP_ASSIGNMENT, PRECEDENCE::TRVIAL, 2,
                 ASSOCIATIVITY::LEFT},

};
static constexpr std::size_t sysy_operator_infos_len =
    sysy_operator_infos.size();
constexpr static auto
sysy_operator_info_map(OPERATION_TYPE type) -> OperatorInfo {
  for (auto &info : sysy_operator_infos) {
    if (type == info.operator_type) {
      return info;
    }
  }
}
constexpr static auto
sysy_operator_assoc(OPERATION_TYPE type) -> ASSOCIATIVITY {
  return sysy_operator_info_map(type).associativity;
}
constexpr static auto sysy_operator_param_number(OPERATION_TYPE type) -> int {
  return sysy_operator_info_map(type).parameter_number;
}
constexpr static auto sysy_operator_prec(OPERATION_TYPE type) -> PRECEDENCE {
  return sysy_operator_info_map(type).precedence;
}

} // namespace compiler::type
namespace compiler {
static auto next_global_id() -> int64_t {
  static int64_t id;
  return ++id;
}
} // namespace compiler
// 偏特化
template <>
struct fmt::formatter<compiler::type::TOKEN_TYPE>
    : fmt::formatter<std::string_view> {
  auto format(compiler::type::TOKEN_TYPE type, fmt::format_context &ctx) const {
    return fmt::formatter<std::string_view>::format(
        compiler::type::token_type_to_string(type), ctx);
  }
};
template <>
struct fmt::formatter<compiler::type::NODE_TYPE>
    : fmt::formatter<std::string_view> {
  auto format(compiler::type::NODE_TYPE type, fmt::format_context &ctx) const {
    return fmt::formatter<std::string_view>::format(
        compiler::type::node_type_to_string(type), ctx);
  }
};
template <>
struct fmt::formatter<compiler::type::VALUE_TYPE>
    : fmt::formatter<std::string_view> {
  auto format(compiler::type::VALUE_TYPE type, fmt::format_context &ctx) const {
    return fmt::formatter<std::string_view>::format(
        compiler::type::value_type_to_string(type), ctx);
  }
};
